@@ -26,7 +26,7 @@ PERFORMANCE OF THIS SOFTWARE.
                                fujiwara@a80.tech.yk.fujitsu.co.jp
 
 ******************************************************************/
-/* $XFree86: xc/lib/X11/imDefLkup.c,v 3.7 2000/11/28 18:49:36 dawes Exp $ */
+/* $XFree86: xc/lib/X11/imDefLkup.c,v 3.9 2001/08/13 21:46:46 dawes Exp $ */
 
 #include <X11/Xatom.h>
 #define  NEED_EVENTS
@@ -898,10 +898,6 @@ _XimError(im, ic, error_code, detail_length, type, detail)
     return True;
 }
 
-#ifndef MAXINT
-#define MAXINT		(~((unsigned int)1 << ((8 * sizeof(int)) - 1)))
-#endif /* !MAXINT */
-
 Private int
 _Ximctsconvert(conv, from, from_len, to, to_len, state)
     XlcConv	 conv;
@@ -919,6 +915,7 @@ _Ximctsconvert(conv, from, from_len, to, to_len, state)
     int		 to_cnvlen;
     char	*from_buf;
     char	*to_buf;
+    char	 scratchbuf[BUFSIZ];
     Status	 tmp_state;
 
     if (!state)
@@ -929,6 +926,10 @@ _Ximctsconvert(conv, from, from_len, to, to_len, state)
 	return 0;
     }
 
+    /* Reset the converter.  The CompoundText at 'from' starts in
+       initial state.  */
+    _XlcResetConverter(conv);
+
     if (to && to_len) {
 	from_left = from_len;
 	to_left = to_len;
@@ -962,10 +963,10 @@ _Ximctsconvert(conv, from, from_len, to, to_len, state)
     from_left = from_len;
     from_cnvlen = 0;
     to_cnvlen = 0;
-    to_buf = NULL;
     for (;;) {
 	from_savelen = from_left;
-	to_left = MAXINT;
+	to_buf = scratchbuf;
+	to_left = BUFSIZ;
 	from_buf = &from[from_cnvlen];
 	if (_XlcConvert(conv, (XPointer *)&from_buf, &from_left,
 				 (XPointer *)&to_buf, &to_left, NULL, 0) < 0) {
@@ -973,7 +974,7 @@ _Ximctsconvert(conv, from, from_len, to, to_len, state)
 	    return 0;
 	}
 	from_cnvlen += (from_savelen - from_left);
-	to_cnvlen += (MAXINT - to_left);
+	to_cnvlen += (BUFSIZ - to_left);
 	if (from_left == 0) {
 	    if (to_cnvlen > 0)
 		*state = XBufferOverflow;
@@ -1017,6 +1018,7 @@ _Ximctstowcs(xim, from, from_len, to, to_len, state)
     int		 to_cnvlen;
     char	*from_buf;
     wchar_t	*to_buf;
+    wchar_t	 scratchbuf[BUFSIZ];
     Status	 tmp_state;
 
     if (!state)
@@ -1027,6 +1029,10 @@ _Ximctstowcs(xim, from, from_len, to, to_len, state)
 	return 0;
     }
 
+    /* Reset the converter.  The CompoundText at 'from' starts in
+       initial state.  */
+    _XlcResetConverter(conv);
+
     if (to && to_len) {
 	from_left = from_len;
 	to_left = to_len;
@@ -1060,10 +1066,10 @@ _Ximctstowcs(xim, from, from_len, to, to_len, state)
     from_left = from_len;
     from_cnvlen = 0;
     to_cnvlen = 0;
-    to_buf = (wchar_t *)NULL;
     for (;;) {
 	from_savelen = from_left;
-	to_left = MAXINT;
+	to_buf = scratchbuf;
+	to_left = BUFSIZ * sizeof(wchar_t);
 	from_buf = &from[from_cnvlen];
 	if (_XlcConvert(conv, (XPointer *)&from_buf, &from_left,
 				 (XPointer *)&to_buf, &to_left, NULL, 0) < 0) {
@@ -1071,7 +1077,7 @@ _Ximctstowcs(xim, from, from_len, to, to_len, state)
 	    return 0;
 	}
 	from_cnvlen += (from_savelen - from_left);
-	to_cnvlen += (MAXINT - to_left);
+	to_cnvlen += (BUFSIZ * sizeof(wchar_t) - to_left);
 	if (from_left == 0) {
 	    if (to_cnvlen > 0)
 		*state = XBufferOverflow;
