@@ -24,13 +24,11 @@
  THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
  ********************************************************/
-/* $XFree86: xc/lib/xkbfile/xkbconfig.c,v 3.4 2000/06/14 18:20:33 dawes Exp $ */
+/* $XFree86: xc/lib/xkbfile/xkbconfig.c,v 3.7 2001/11/30 12:11:51 eich Exp $ */
 
 #include <stdio.h>
 #include <ctype.h>
-#ifndef X_NOT_STDC_ENV
 #include <stdlib.h>
-#endif
 
 #include <X11/Xfuncs.h>
 
@@ -880,7 +878,8 @@ unsigned		what;
 		}
 	    }
 	    *pival= val.ival * sign;
-	    rtrn->defined|= XkbCF_AccessXTimeout;
+	    if (field->field_id == _XkbCF_AccessXTimeout)
+	        rtrn->defined|=XkbCF_AccessXTimeout;
 	    tok= XkbCFScan(file,&val,rtrn);
 	    if ((tok!=XkbCF_EOL)&&(tok!=XkbCF_Semi)&&(tok!=XkbCF_EOF)) {
 		rtrn->error= XkbCF_ExpectedEOS;
@@ -1076,7 +1075,7 @@ unsigned int	mask;
 	ctrls->mk_max_speed= rtrn->mk_max_speed;
     if (rtrn->mk_curve>0)
 	ctrls->mk_curve= rtrn->mk_curve;
-    if (rtrn->defined&XkbCF_AccessXTimeout)
+    if (rtrn->defined&XkbCF_AccessXTimeout && rtrn->ax_timeout > 0)
 	ctrls->ax_timeout= rtrn->ax_timeout;
 
     /* any value set to both off and on is reset to ignore */
