@@ -24,7 +24,7 @@
  THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
  ********************************************************/
-/* $XFree86: xc/lib/xkbfile/xkmread.c,v 1.3 2000/10/28 00:34:02 dawes Exp $ */
+/* $XFree86: xc/lib/xkbfile/xkmread.c,v 1.6 2002/02/13 22:09:42 herrb Exp $ */
 
 #include <stdio.h>
 
@@ -33,9 +33,7 @@
 
 #ifndef XKB_IN_SERVER
 
-#ifndef X_NOT_STDC_ENV
 #include <stdlib.h>
-#endif
 #include <X11/Xlib.h>
 #include <X11/keysym.h>
 
@@ -173,7 +171,7 @@ XkmGetCARD32(file,pNRead)
     int *	pNRead;
 #endif
 {
-unsigned long	val;
+CARD32	val;
 
     if ((fread(&val,4,1,file)==1)&&(pNRead))
 	(*pNRead)+= 4;
