@@ -1,10 +1,14 @@
-/* $Xorg: osPexFont.c,v 1.3 2000/08/17 19:47:20 cpqbld Exp $ */
+/* $Xorg: osPexFont.c,v 1.4 2001/02/09 02:04:19 xorgcvs Exp $ */
 
 /*
 
 Copyright 1989, 1990, 1991, 1998  The Open Group
 
-All Rights Reserved.
+Permission to use, copy, modify, distribute, and sell this software and its
+documentation for any purpose is hereby granted without fee, provided that
+the above copyright notice appear in all copies and that both that
+copyright notice and this permission notice appear in supporting
+documentation.
 
 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.
@@ -42,7 +46,7 @@ ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 SOFTWARE.
 
 */
-/* $XFree86: xc/programs/Xserver/PEX5/ospex/osPexFont.c,v 3.16 2000/08/10 17:40:30 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/PEX5/ospex/osPexFont.c,v 3.19 2002/05/31 18:45:53 dawes Exp $ */
 
 #ifdef WIN32
 #define _WILLWINSOCK_
@@ -131,7 +135,7 @@ pex_get_font_directory_path()
 	       (char *)xalloc((unsigned long)(1+strlen(getenv("PEX_FONTPATH"))));
 	    strcpy(font_dir_path, getenv("PEX_FONTPATH"));
 	} else {
-#ifndef __EMX__
+#ifndef __UNIXOS2__
 	    font_dir_path =
 		(char *)xalloc((unsigned long)(1+strlen(PEX_DEFAULT_FONTPATH)));
 	    strcpy(font_dir_path, PEX_DEFAULT_FONTPATH);
