@@ -1,9 +1,13 @@
-/* $Xorg: security.c,v 1.3 2000/08/17 19:47:58 cpqbld Exp $ */
+/* $Xorg: security.c,v 1.4 2001/02/09 02:04:32 xorgcvs Exp $ */
 /*
 
 Copyright 1996, 1998  The Open Group
 
-All Rights Reserved.
+Permission to use, copy, modify, distribute, and sell this software and its
+documentation for any purpose is hereby granted without fee, provided that
+the above copyright notice appear in all copies and that both that
+copyright notice and this permission notice appear in supporting
+documentation.
 
 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.
@@ -20,7 +24,7 @@ used in advertising or otherwise to promote the sale, use or other dealings
 in this Software without prior written authorization from The Open Group.
 
 */
-/* $XFree86: xc/programs/Xserver/Xext/security.c,v 1.5 2000/08/10 17:40:30 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/Xext/security.c,v 1.11 2002/05/31 18:45:53 dawes Exp $ */
 
 #include "dixstruct.h"
 #include "extnsionst.h"
@@ -79,9 +83,6 @@ int (*SwappedUntrustedProcVector[256])(
 #endif
 );
 
-extern int ProcBadRequest();
-
-
 /* SecurityAudit
  *
  * Arguments:
@@ -147,7 +148,7 @@ SecurityDeleteAuthorization(value, id)
 
     /* send revoke events */
 
-    while (pEventClient = pAuth->eventClients)
+    while ((pEventClient = pAuth->eventClients))
     {
 	/* send revocation event event */
 	ClientPtr client = rClient(pEventClient);
@@ -315,7 +316,7 @@ static int
 ProcSecurityQueryVersion(client)
     ClientPtr client;
 {
-    REQUEST(xSecurityQueryVersionReq);
+    /* REQUEST(xSecurityQueryVersionReq); */
     xSecurityQueryVersionReply 	rep;
 
     /* paranoia: this "can't happen" because this extension is hidden
@@ -392,7 +393,6 @@ ProcSecurityGenerateAuthorization(client)
     Bool removeAuth = FALSE;	/* if bailout, call RemoveAuthorization? */
     SecurityAuthorizationPtr pAuth = NULL;  /* auth we are creating */
     int err;			/* error to return from this function */
-    int status;			/* return value from os functions */
     XID authId;			/* authorization ID assigned by os layer */
     xSecurityGenerateAuthorizationReply rep; /* reply struct */
     unsigned int trustLevel;    /* trust level of new auth */
@@ -785,7 +785,7 @@ SecurityCheckDeviceAccess(client, dev, fromRequest)
     Bool untrusted_got_event;
     Bool found_event_window;
     Mask eventmask;
-    int reqtype;
+    int reqtype = 0;
 
     /* trusted clients always allowed to do anything */
     if (client->trustLevel == XSecurityClientTrusted)
@@ -1358,7 +1358,7 @@ SecurityFreePropertyAccessList()
     }
 } /* SecurityFreePropertyAccessList */
 
-#ifndef __EMX__
+#ifndef __UNIXOS2__
 #define SecurityIsWhitespace(c) ( (c == ' ') || (c == '\t') || (c == '\n') )
 #else
 #define SecurityIsWhitespace(c) ( (c == ' ') || (c == '\t') || (c == '\n') || (c == '\r') )
@@ -1441,7 +1441,6 @@ SecurityParsePropertyAccessRule(p)
     char action = SecurityDefaultAction;
     char readAction, writeAction, destroyAction;
     PropertyAccessPtr pacl, prev, cur;
-    ATOM atom;
     char *mustHaveProperty = NULL;
     char *mustHaveValue = NULL;
     Bool invalid;
@@ -1641,7 +1640,7 @@ SecurityLoadPropertyAccessList()
     if (!SecurityPolicyFile)
 	return;
 
-#ifndef __EMX__
+#ifndef __UNIXOS2__
     f = fopen(SecurityPolicyFile, "r");
 #else
     f = fopen((char*)__XOS2RedirRoot(SecurityPolicyFile), "r");
@@ -1973,7 +1972,7 @@ SecurityExtensionInit()
     SecurityEventBase = extEntry->eventBase;
 
     EventSwapVector[SecurityEventBase + XSecurityAuthorizationRevoked] =
-	SwapSecurityAuthorizationRevokedEvent;
+	(EventSwapPtr)SwapSecurityAuthorizationRevokedEvent;
 
     /* initialize untrusted proc vectors */
 
