@@ -1,9 +1,13 @@
 /*
- * $Xorg: cfb8line.c,v 1.3 2000/08/17 19:48:13 cpqbld Exp $
+ * $Xorg: cfb8line.c,v 1.4 2001/02/09 02:04:37 xorgcvs Exp $
  *
 Copyright 1990, 1998  The Open Group
 
-All Rights Reserved.
+Permission to use, copy, modify, distribute, and sell this software and its
+documentation for any purpose is hereby granted without fee, provided that
+the above copyright notice appear in all copies and that both that
+copyright notice and this permission notice appear in supporting
+documentation.
 
 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.
@@ -21,7 +25,7 @@ in this Software without prior written authorization from The Open Group.
  *
  * Author:  Keith Packard, MIT X Consortium
  *
- * $XFree86: xc/programs/Xserver/cfb/cfb8line.c,v 3.14 2001/01/17 22:36:34 dawes Exp $
+ * $XFree86: xc/programs/Xserver/cfb/cfb8line.c,v 3.18 2002/09/18 17:11:47 tsi Exp $
  * Jeff Anton'x fixes: cfb8line.c   97/02/07
  */
 
@@ -271,7 +275,7 @@ FUNC_NAME(cfb8LineSS1Rect) (pDrawable, pGC, mode, npt, pptInit, pptInitOrig,
     int	npt;		/* number of points */
     DDXPointPtr pptInit, pptInitOrig;
     int	*x1p, *y1p, *x2p, *y2p;
-#endif /* POLYSEGEMENT */
+#endif /* POLYSEGMENT */
 {
     register long   e;
     register int    y1_or_e1;
@@ -293,8 +297,8 @@ FUNC_NAME(cfb8LineSS1Rect) (pDrawable, pGC, mode, npt, pptInit, pptInitOrig,
 #else
     register int    c2;
 #endif
-#ifndef ORIGIN
-    register int _x1, _y1, _x2, _y2;	/* only used for CoordModePrevious */
+#if !defined(ORIGIN) && !defined(POLYSEGMENT)
+    register int _x1 = 0, _y1 = 0, _x2 = 0, _y2 = 0;
     int extents_x1, extents_y1, extents_x2, extents_y2;
 #endif /* !ORIGIN */
 #ifndef PREVIOUS
@@ -329,7 +333,6 @@ FUNC_NAME(cfb8LineSS1Rect) (pDrawable, pGC, mode, npt, pptInit, pptInitOrig,
 #if PSZ == 24
     int xBase;     /* x of addr */
     int xOffset;   /* x of addrp */
-    int xOffset_t; /* x of t */
     PixelType   *addrLineEnd;
     char *addrb;
     int stepmajor3, stepminor3, majordx, minordx;
@@ -730,16 +733,19 @@ FUNC_NAME(cfb8LineSS1Rect) (pDrawable, pGC, mode, npt, pptInit, pptInitOrig,
 		  *addrp   = piQxelXor[2];
 		  break;
 		case 1:
-		  *addrp++ = ((*addrp) & 0xFFFFFF) | (piQxelXor[0] & 0xFF000000);
-		  *addrp++ = piQxelXor[1];
-		  *addrp   = piQxelXor[2];
+		  *addrp = ((*addrp) & 0xFFFFFF) | (piQxelXor[0] & 0xFF000000);
+		  addrp++;
+		  *addrp = piQxelXor[1];
+		  addrp++;
+		  *addrp = piQxelXor[2];
 		  break;
 		case 2:
-		  *addrp++ =((*addrp) & 0xFFFF) | (piQxelXor[1] & 0xFFFF0000);
+		  *addrp = ((*addrp) & 0xFFFF) | (piQxelXor[1] & 0xFFFF0000);
+		  addrp++;
 		  *addrp = piQxelXor[2];
 		  break;
 		case 3:
-		  *addrp =((*addrp) & 0xFF) | (piQxelXor[2] & 0xFFFFFF00);
+		  *addrp = ((*addrp) & 0xFF) | (piQxelXor[2] & 0xFFFFFF00);
 		  break;
 		}
 		break;
@@ -751,12 +757,15 @@ FUNC_NAME(cfb8LineSS1Rect) (pDrawable, pGC, mode, npt, pptInit, pptInitOrig,
 		  *addrp = ((*addrp) & 0xFFFFFF00) | (piQxelXor[2] & 0xFF);
 		  break;
 		case 1:
-		  *addrp++ = ((*addrp) & 0xFFFFFF) | (piQxelXor[0] & 0xFF000000);
-		  *addrp++ = piQxelXor[1];
+		  *addrp = ((*addrp) & 0xFFFFFF) | (piQxelXor[0] & 0xFF000000);
+		  addrp++;
+		  *addrp = piQxelXor[1];
+		  addrp++;
 		  *addrp = ((*addrp) & 0xFFFFFF00) | (piQxelXor[2] & 0xFF);
 		  break;
 		case 2:
-		  *addrp++ =((*addrp) & 0xFFFF) | (piQxelXor[1] & 0xFFFF0000);
+		  *addrp = ((*addrp) & 0xFFFF) | (piQxelXor[1] & 0xFFFF0000);
+		  addrp++;
 		  *addrp = ((*addrp) & 0xFFFFFF00) | (piQxelXor[2] & 0xFF);
 		  break;
 		}
@@ -765,12 +774,14 @@ FUNC_NAME(cfb8LineSS1Rect) (pDrawable, pGC, mode, npt, pptInit, pptInitOrig,
 		switch(leftIndex){
 /*
 		case 2:
-		  *addrp++ = ((*addrp) & 0xFFFF) | (piQxelXor[1] & 0xFFFF0000);
+		  *addrp = ((*addrp) & 0xFFFF) | (piQxelXor[1] & 0xFFFF0000);
+		  addrp++;
 		  *addrp = ((*addrp) & 0xFFFFFF00) | (piQxelXor[2] & 0xFF);
 		  break;
 */
 		case 1:
-		  *addrp++ = ((*addrp) & 0xFFFFFF) | (piQxelXor[0] & 0xFF000000);
+		  *addrp = ((*addrp) & 0xFFFFFF) | (piQxelXor[0] & 0xFF000000);
+		  addrp++;
 		  *addrp = ((*addrp) & 0xFFFF0000) | (piQxelXor[1] & 0xFFFF);
 		  break;
 		case 0:
@@ -785,7 +796,8 @@ FUNC_NAME(cfb8LineSS1Rect) (pDrawable, pGC, mode, npt, pptInit, pptInitOrig,
 		}
 /*
 		else{
-		*addrp++ =  ((*addrp) & 0xFFFFFF) | (piQxelXor[0] & 0xFF000000);
+		*addrp =  ((*addrp) & 0xFFFFFF) | (piQxelXor[0] & 0xFF000000);
+		addrp++;
 		*addrp =  ((*addrp) & 0xFFFF0000) | (piQxelXor[1] & 0xFFFF);
 		}
 */
@@ -801,16 +813,22 @@ FUNC_NAME(cfb8LineSS1Rect) (pDrawable, pGC, mode, npt, pptInit, pptInitOrig,
 		  case 0:
 		    break;
 		  case 1:
-		    *addrp++ = ((*addrp) & 0xFFFFFF) | (piQxelXor[0] & 0xFF000000);
-		    *addrp++ = piQxelXor[1];
-		    *addrp++ = piQxelXor[2];
+		    *addrp = ((*addrp) & 0xFFFFFF) | (piQxelXor[0] & 0xFF000000);
+		    addrp++;
+		    *addrp = piQxelXor[1];
+		    addrp++;
+		    *addrp = piQxelXor[2];
+		    addrp++;
 		    break;
 		  case 2:
-		    *addrp++ = ((*addrp) & 0xFFFF) | (piQxelXor[1] & 0xFFFF0000);
-		    *addrp++ = piQxelXor[2];
+		    *addrp = ((*addrp) & 0xFFFF) | (piQxelXor[1] & 0xFFFF0000);
+		    addrp++;
+		    *addrp = piQxelXor[2];
+		    addrp++;
 		    break;
 		  case 3:
-		    *addrp++ = ((*addrp) & 0xFF) | (piQxelXor[2] & 0xFFFFFF00);
+		    *addrp = ((*addrp) & 0xFF) | (piQxelXor[2] & 0xFFFFFF00);
+		    addrp++;
 		    break;
 		  }
 		  while(nlmiddle--){
@@ -843,20 +861,23 @@ FUNC_NAME(cfb8LineSS1Rect) (pDrawable, pGC, mode, npt, pptInit, pptInitOrig,
 			| (piQxelXor[0] & 0xFFFFFF & e3);
 		      break;
 		    case 1:
-		      *addrp++ = ((*addrp) & (0xFFFFFF | ~(e3<<24)))
-				  | (piQxelXor[0] & 0xFF000000 & (e3<<24));
+		      *addrp = ((*addrp) & (0xFFFFFF | ~(e3<<24)))
+				  + (piQxelXor[0] & 0xFF000000 & (e3<<24));
+		      addrp++;
 		      *addrp = ((*addrp) & (0xFFFF0000|~(e3 >> 8)))
 				  | (piQxelXor[1] & 0xFFFF & (e3 >> 8));
 		      break;
 		    case 2:
-		      *addrp++ = ((*addrp) & (0xFFFF|~(e3 << 16)))
+		      *addrp = ((*addrp) & (0xFFFF|~(e3 << 16)))
 				  | (piQxelXor[1] & 0xFFFF0000 & (e3 << 16));
+		      addrp++;
 		      *addrp = ((*addrp) & (0xFFFFFF00|~(e3>>16)))
 				  | (piQxelXor[2] & 0xFF & (e3 >> 16));
 		      break;
 		    case 3:
-		      *addrp++ = ((*addrp) & (0xFF|~(e3<<8)))
+		      *addrp = ((*addrp) & (0xFF|~(e3<<8)))
 				  | (piQxelXor[2] & 0xFFFFFF00 & (e3<<8));
+		      addrp++;
 		      break;
 		    }
 		  }
@@ -932,9 +953,7 @@ FUNC_NAME(cfb8LineSS1Rect) (pDrawable, pGC, mode, npt, pptInit, pptInitOrig,
 #endif /* !ORIGIN */	    
 	return ((DDXPointPtr) ppt - pptInit) - 1;
     }
-#endif /* POLYSEGMENT */
 
-#ifndef POLYSEGMENT
 # ifndef ORIGIN
 #  define C2  c2
 # else
