@@ -21,7 +21,7 @@
  * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
  * PERFORMANCE OF THIS SOFTWARE.
  */
-/* $XFree86: xc/programs/Xserver/fb/fbbltone.c,v 1.9 2000/05/06 21:09:32 keithp Exp $ */
+/* $XFree86: xc/programs/Xserver/fb/fbbltone.c,v 1.13 2002/02/23 00:42:07 keithp Exp $ */
 
 #include "fb.h"
 
@@ -58,15 +58,15 @@
     
 #ifndef FBNOPIXADDR
     
-#define LaneCases1(n,a)	    case n: FbLaneCase(n,a); break;
-#define LaneCases2(n,a)	    LaneCases1(n,a) LaneCases1(n+1,a)
-#define LaneCases4(n,a)	    LaneCases2(n,a) LaneCases2(n+2,a)
-#define LaneCases8(n,a)	    LaneCases4(n,a) LaneCases4(n+4,a)
-#define LaneCases16(n,a)    LaneCases8(n,a) LaneCases8(n+8,a)
-#define LaneCases32(n,a)    LaneCases16(n,a) LaneCases16(n+16,a)
-#define LaneCases64(n,a)    LaneCases32(n,a) LaneCases32(n+32,a)
-#define LaneCases128(n,a)   LaneCases64(n,a) LaneCases64(n+64,a)
-#define LaneCases256(n,a)   LaneCases128(n,a) LaneCases128(n+128,a)
+#define LaneCases1(n,a)	    case n: (void)FbLaneCase(n,a); break
+#define LaneCases2(n,a)	    LaneCases1(n,a); LaneCases1(n+1,a)
+#define LaneCases4(n,a)	    LaneCases2(n,a); LaneCases2(n+2,a)
+#define LaneCases8(n,a)	    LaneCases4(n,a); LaneCases4(n+4,a)
+#define LaneCases16(n,a)    LaneCases8(n,a); LaneCases8(n+8,a)
+#define LaneCases32(n,a)    LaneCases16(n,a); LaneCases16(n+16,a)
+#define LaneCases64(n,a)    LaneCases32(n,a); LaneCases32(n+32,a)
+#define LaneCases128(n,a)   LaneCases64(n,a); LaneCases64(n+64,a)
+#define LaneCases256(n,a)   LaneCases128(n,a); LaneCases128(n+128,a)
     
 #if FB_SHIFT == 6
 #define LaneCases(a)	    LaneCases256(0,a)
@@ -151,7 +151,7 @@ fbBltOne (FbStip    *src,
     int		    unitsPerSrc;		/* src patterns per FbStip */
     int		    leftShift, rightShift;	/* align source with dest */
     FbBits	    startmask, endmask;		/* dest scanline masks */
-    FbStip	    bits, bitsLeft, bitsRight;	/* source bits */
+    FbStip	    bits=0, bitsLeft, bitsRight;/* source bits */
     FbStip	    left;
     FbBits	    mask;
     int		    nDst;			/* dest longwords (w.o. end) */
@@ -161,7 +161,7 @@ fbBltOne (FbStip    *src,
     Bool	    copy;			/* accelerate dest-invariant */
     Bool	    transparent;		/* accelerate 0 nop */
     int		    srcinc;			/* source units consumed */
-    Bool	    endNeedsLoad;		/* need load for endmask */
+    Bool	    endNeedsLoad = FALSE;	/* need load for endmask */
 #ifndef FBNOPIXADDR
     CARD8	    *fbLane;
 #endif
@@ -505,12 +505,21 @@ const FbBits	fbStipple24Bits[3][1 << FbStip24Len] = {
 #endif
 
 #if BITMAP_BIT_ORDER == LSBFirst
-#define FbMergeStip24Bits(left, right, new) (((left) >> (new)) | \
-					     (right) << (FbStip24Len - (new)))
+
+#define FbMergeStip24Bits(left, right, new) \
+	(FbStipLeft (left, new) | FbStipRight ((right), (FbStip24Len - (new))))
+
+#define FbMergePartStip24Bits(left, right, llen, rlen) \
+	(left | FbStipRight(right, llen))
+
 #else
-#define FbMergeStip24Bits(left, right, new) (((left << new) & \
-					      ((1 << FbStip24Len) - 1)) | \
-					     right)
+
+#define FbMergeStip24Bits(left, right, new) \
+	((FbStipLeft (left, new) & ((1 << FbStip24Len) - 1)) | right)
+
+#define FbMergePartStip24Bits(left, right, llen, rlen) \
+	(FbStipLeft(left, rlen) | right)
+
 #endif
 
 #define fbFirstStipBits(len,stip) {\
@@ -521,7 +530,8 @@ const FbBits	fbStipple24Bits[3][1 << FbStip24Len] = {
 	stip = FbLeftStipBits(bits, remain); \
 	bits = *src++; \
 	__len = (len) - remain; \
-	stip |= FbStipRight(FbLeftStipBits(bits, __len), remain); \
+	stip = FbMergePartStip24Bits(stip, FbLeftStipBits(bits, __len), \
+				     remain, __len); \
 	remain = FB_STIP_UNIT; \
     } \
     bits = FbStipLeft (bits, __len); \
@@ -753,7 +763,7 @@ fbBltPlane (FbBits	    *src,
     FbBits	pm;
     FbBits	srcMask;
     FbBits	srcMaskFirst;
-    FbBits	srcMask0;
+    FbBits	srcMask0 = 0;
     FbBits	srcBits;
     
     FbStip	dstBits;
