@@ -1,4 +1,4 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/common/xf86Mode.c,v 1.38 2001/02/15 20:31:51 eich Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/common/xf86Mode.c,v 1.63 2003/01/22 21:44:09 tsi Exp $ */
 
 /*
  * Copyright (c) 1997,1998 by The XFree86 Project, Inc.
@@ -11,9 +11,11 @@
 
 #include "X.h"
 #include "os.h"
+#include "servermd.h"
 #include "mibank.h"
 #include "xf86.h"
 #include "xf86Priv.h"
+#include "xf86DDC.h"
 
 /*
  * xf86GetNearestClock --
@@ -308,6 +310,8 @@ xf86HandleBuiltinMode(ScrnInfoPtr scrp,
     modep->CrtcVAdjusted   = p->CrtcVAdjusted;
     modep->HSync           = p->HSync;
     modep->VRefresh        = p->VRefresh;
+    modep->Private         = p->Private;
+    modep->PrivSize        = p->PrivSize;
 
     p->prev = modep;
     
@@ -326,7 +330,8 @@ xf86HandleBuiltinMode(ScrnInfoPtr scrp,
  *    scrp         ScrnInfoPtr
  *    modep        pointer to the returned mode, which must have the name
  *                 field filled in.
- *    clockRanges  a list of clock ranges
+ *    clockRanges  a list of clock ranges.   This is optional when all the
+ *                 modes are built-in modes.
  *    strategy     how to decide which mode to use from multiple modes with
  *                 the same name
  *
@@ -357,7 +362,9 @@ xf86LookupMode(ScrnInfoPtr scrp, DisplayModePtr modep,
     int ModePrivFlags = 0;
     ModeStatus status = MODE_NOMODE;
     Bool allowDiv2 = (strategy & LOOKUP_CLKDIV2) != 0;
-    strategy &= ~LOOKUP_CLKDIV2;
+    Bool haveBuiltin;
+
+    strategy &= ~(LOOKUP_CLKDIV2 | LOOKUP_OPTIONAL_TOLERANCES);
 
     /* Some sanity checking */
     if (scrp == NULL || scrp->modePool == NULL ||
@@ -369,16 +376,13 @@ xf86LookupMode(ScrnInfoPtr scrp, DisplayModePtr modep,
 	ErrorF("xf86LookupMode: called with invalid modep\n");
 	return MODE_ERROR;
     }
-    if (clockRanges == NULL) {
-	ErrorF("xf86LookupMode: called with invalid clockRanges\n");
-	return MODE_ERROR;
-    }
     for (cp = clockRanges; cp != NULL; cp = cp->next) {
 	/* DivFactor and MulFactor must be > 0 */
 	cp->ClockDivFactor = max(1, cp->ClockDivFactor);
 	cp->ClockMulFactor = max(1, cp->ClockMulFactor);
     }
 
+    haveBuiltin = FALSE;
     /* Scan the mode pool for matching names */
     for (p = scrp->modePool; p != NULL; p = p->next) {
 	if (strcmp(p->name, modep->name) == 0) {
@@ -388,9 +392,13 @@ xf86LookupMode(ScrnInfoPtr scrp, DisplayModePtr modep,
 	     * Since built-in modes always come before user specified
 	     * modes it will always be found first.  
 	     */
-	    if (p->type & M_T_BUILTIN) 
-		return xf86HandleBuiltinMode(scrp, p,modep, clockRanges,
-					     allowDiv2);
+	    if (p->type & M_T_BUILTIN) {
+		haveBuiltin = TRUE;
+	    }
+
+	    if (haveBuiltin && !(p->type & M_T_BUILTIN))
+		continue;
+
 	    /* Skip over previously rejected modes */
 	    if (p->status != MODE_OK) {
 		if (!found)
@@ -402,6 +410,11 @@ xf86LookupMode(ScrnInfoPtr scrp, DisplayModePtr modep,
 	    if (p->prev)
 		continue;
 
+	    if (p->type & M_T_BUILTIN) {
+		return xf86HandleBuiltinMode(scrp, p,modep, clockRanges,
+					     allowDiv2);
+	    }
+
 	    /* Check clock is in range */
 	    cp = xf86FindClockRangeForMode(clockRanges, p);
 	    if (cp == NULL) {
@@ -574,6 +587,8 @@ xf86LookupMode(ScrnInfoPtr scrp, DisplayModePtr modep,
     modep->CrtcVAdjusted	= bestMode->CrtcVAdjusted;
     modep->HSync		= bestMode->HSync;
     modep->VRefresh		= bestMode->VRefresh;
+    modep->Private		= bestMode->Private;
+    modep->PrivSize		= bestMode->PrivSize;
 
     bestMode->prev = modep;
 
@@ -668,6 +683,36 @@ xf86CheckModeForMonitor(DisplayModePtr mode, MonPtr monitor)
 	return MODE_ERROR;
     }
 
+#ifdef DEBUG
+    ErrorF("xf86CheckModeForMonitor(%p %s, %p %s)\n",
+	   mode, mode->name, monitor, monitor->id);
+#endif
+
+    if (monitor->DDC) {
+	xf86MonPtr DDC = (xf86MonPtr)(monitor->DDC);
+	struct detailed_monitor_section* detMon;
+	struct monitor_ranges *mon_range;
+	int i;
+
+	mon_range = NULL;
+	for (i = 0; i < 4; i++) {
+	    detMon = &DDC->det_mon[i];
+	    if(detMon->type == DS_RANGES) {
+		mon_range = &detMon->section.ranges;
+	    }
+	}
+	if (mon_range) {
+	    /* mode->Clock in kHz, DDC in MHz */
+	    if (mon_range->max_clock < 2550 &&
+		 mode->Clock / 1000.0 > mon_range->max_clock) {
+		xf86Msg(X_WARNING,
+		   "(%s,%s) mode clock %gMHz exceeds DDC maximum %dMHz\n",
+		   mode->name, monitor->id,
+		   mode->Clock/1000.0, mon_range->max_clock);
+	    }
+	}
+    }
+
     /* Some basic mode validity checks */
     if (0 >= mode->HDisplay || mode->HDisplay > mode->HSyncStart ||
 	mode->HSyncStart >= mode->HSyncEnd || mode->HSyncEnd >= mode->HTotal)
@@ -716,6 +761,55 @@ xf86CheckModeForMonitor(DisplayModePtr mode, MonPtr monitor)
     return MODE_OK;
 }
 
+/*
+ * xf86CheckModeSize
+ *
+ * An internal routine to check if a mode fits in video memory.  This tries to
+ * avoid overflows that would otherwise occur when video memory size is greater
+ * than 256MB.
+ */
+static Bool
+xf86CheckModeSize(ScrnInfoPtr scrp, int w, int x, int y)
+{
+    int bpp = scrp->fbFormat.bitsPerPixel,
+	pad = scrp->fbFormat.scanlinePad;
+    int lineWidth, lastWidth;
+
+    if (scrp->depth == 4)
+	pad *= 4;		/* 4 planes */
+
+    /* Sanity check */
+    if ((w < 0) || (x < 0) || (y <= 0))
+	return FALSE;
+
+    lineWidth = (((w * bpp) + pad - 1) / pad) * pad;
+    lastWidth = x * bpp;
+
+    /*
+     * At this point, we need to compare
+     *
+     *	(lineWidth * (y - 1)) + lastWidth
+     *
+     * against
+     *
+     *	scrp->videoRam * (1024 * 8)
+     *
+     * These are bit quantities.  To avoid overflows, do the comparison in
+     * terms of BITMAP_SCANLINE_PAD units.  This assumes BITMAP_SCANLINE_PAD
+     * is a power of 2.  We currently use 32, which limits us to a video
+     * memory size of 8GB.
+     */
+
+    lineWidth = (lineWidth + (BITMAP_SCANLINE_PAD - 1)) / BITMAP_SCANLINE_PAD;
+    lastWidth = (lastWidth + (BITMAP_SCANLINE_PAD - 1)) / BITMAP_SCANLINE_PAD;
+
+    if ((lineWidth * (y - 1) + lastWidth) >
+	(scrp->videoRam * ((1024 * 8) / BITMAP_SCANLINE_PAD)))
+	return FALSE;
+
+    return TRUE;
+}
+
 /*
  * xf86InitialCheckModeForDriver
  *
@@ -753,12 +847,17 @@ xf86InitialCheckModeForDriver(ScrnInfoPtr scrp, DisplayModePtr mode,
     int i, needDiv2;
 
     /* Sanity checks */
-    if (!scrp || !mode || !clockRanges) {
+    if (!scrp || !mode /*|| !clockRanges*/) {
 	ErrorF("xf86InitialCheckModeForDriver: "
 		"called with invalid parameters\n");
 	return MODE_ERROR;
     }
 
+#ifdef DEBUG
+    ErrorF("xf86InitialCheckModeForDriver(%p, %p %s, %p, 0x%x, %d, %d, %d)\n",
+	   scrp, mode, mode->name , clockRanges, strategy, maxPitch,  virtualX, virtualY);
+#endif
+
     /* Some basic mode validity checks */
     if (0 >= mode->HDisplay || mode->HDisplay > mode->HSyncStart ||
 	mode->HSyncStart >= mode->HSyncEnd || mode->HSyncEnd >= mode->HTotal)
@@ -768,8 +867,8 @@ xf86InitialCheckModeForDriver(ScrnInfoPtr scrp, DisplayModePtr mode,
 	mode->VSyncStart >= mode->VSyncEnd || mode->VSyncEnd >= mode->VTotal)
 	return MODE_V_ILLEGAL;
 
-    if (mode->HDisplay * mode->VDisplay * scrp->fbFormat.bitsPerPixel >
-        scrp->videoRam * (1024 * 8))
+    if (!xf86CheckModeSize(scrp, mode->HDisplay, mode->HDisplay,
+				 mode->VDisplay))
         return MODE_MEM;
 
     if (maxPitch > 0 && mode->HDisplay > maxPitch)
@@ -837,7 +936,8 @@ xf86InitialCheckModeForDriver(ScrnInfoPtr scrp, DisplayModePtr mode,
 	return MODE_ERROR;
     }
 
-    mode->HSync = (float)mode->SynthClock / (float)mode->CrtcHTotal;
+    if (mode->HSync <= 0.0)
+	mode->HSync = (float)mode->SynthClock / (float)mode->CrtcHTotal;
     if (monitor->nHsync > 0) {
 	/* Check hsync against the allowed ranges */
 	for (i = 0; i < monitor->nHsync; i++)
@@ -850,8 +950,9 @@ xf86InitialCheckModeForDriver(ScrnInfoPtr scrp, DisplayModePtr mode,
 	    return MODE_HSYNC;
     }
 
-    mode->VRefresh = (mode->SynthClock * 1000.0) /
-	(mode->CrtcHTotal * mode->CrtcVTotal);
+    if (mode->VRefresh <= 0.0)
+	mode->VRefresh = (mode->SynthClock * 1000.0) /
+	    (mode->CrtcHTotal * mode->CrtcVTotal);
     if (monitor->nVrefresh > 0) {
 	/* Check vrefresh against the allowed ranges */
 	for (i = 0; i < monitor->nVrefresh; i++)
@@ -920,10 +1021,6 @@ xf86CheckModeForDriver(ScrnInfoPtr scrp, DisplayModePtr mode, int flags)
 	ErrorF("xf86CheckModeForDriver: called with invalid modep\n");
 	return MODE_ERROR;
     }
-    if (scrp->clockRanges == NULL) {
-	ErrorF("xf86CheckModeForDriver: called with invalid clockRanges\n");
-	return MODE_ERROR;
-    }
 
     /* Check the mode size */
     if (mode->HDisplay > scrp->virtualX)
@@ -1088,12 +1185,23 @@ xf86ValidateModes(ScrnInfoPtr scrp, DisplayModePtr availModes,
     ModeStatus status;
     int linePitch = -1, virtX = 0, virtY = 0;
     int newLinePitch, newVirtX, newVirtY;
-    int pixelArea = scrp->videoRam * (1024 * 8);	/* in bits */
     int modeSize;					/* in pixels */
-    int bitsPerPixel, pixmapPad;
+    Bool validateAllDefaultModes;
+    Bool userModes = FALSE;
+    int saveType;
     PixmapFormatRec *BankFormat;
     ClockRangePtr cp;
     ClockRangesPtr storeClockRanges;
+    struct monitor_ranges *mon_range = NULL;
+
+#ifdef DEBUG
+    ErrorF("xf86ValidateModes(%p, %p, %p, %p,\n\t\t  %p, %d, %d, %d, %d, %d, %d, %d, %d, 0x%x)\n",
+	   scrp, availModes, modeNames, clockRanges,
+	   linePitches, minPitch, maxPitch, pitchInc,
+	   minHeight, maxHeight, virtualX, virtualY,
+	   apertureSize, strategy
+	   );
+#endif
 
     /* Some sanity checking */
     if (scrp == NULL || scrp->name == NULL || !scrp->monitor ||
@@ -1109,15 +1217,65 @@ xf86ValidateModes(ScrnInfoPtr scrp, DisplayModePtr availModes,
 	ErrorF("xf86ValidateModes: called with invalid pitchInc\n");
 	return -1;
     }
-    if (clockRanges == NULL) {
-	ErrorF("xf86ValidateModes: called with invalid clockRanges\n");
-	return -1;
-    }
     if ((virtualX > 0) != (virtualY > 0)) {
 	ErrorF("xf86ValidateModes: called with invalid virtual resolution\n");
 	return -1;
     }
 
+    /* Probe monitor so that we can enforce/warn about its limits */
+    if (scrp->monitor->DDC) {
+	MonPtr monitor = scrp->monitor;
+	xf86MonPtr DDC = (xf86MonPtr)(scrp->monitor->DDC);
+	struct detailed_monitor_section* detMon;
+	int i;
+
+	for (i = 0; i < 4; i++) {
+	    detMon = &DDC->det_mon[i];
+	    if(detMon->type == DS_RANGES) {
+		mon_range = &detMon->section.ranges;
+	    }
+	}
+	if (mon_range) {
+
+#ifdef DEBUG
+	    ErrorF("DDC - Max clock %d, Hsync %d-%d kHz - Vrefresh %d-%d Hz\n",
+	           mon_range->max_clock, mon_range->min_h, mon_range->max_h,
+	           mon_range->min_v, mon_range->max_v );
+#endif
+
+#define DDC_SYNC_TOLERANCE SYNC_TOLERANCE
+	    if (monitor->nHsync > 0) {
+		for (i = 0; i < monitor->nHsync; i++) {
+		    if ((1.0 - DDC_SYNC_TOLERANCE) * mon_range->min_h >
+				monitor->hsync[i].lo ||
+			(1.0 + DDC_SYNC_TOLERANCE) * mon_range->max_h <
+				monitor->hsync[i].hi) {
+			xf86DrvMsg(scrp->scrnIndex, X_WARNING,
+			  "config file hsync range %g-%gkHz not within DDC "
+			  "hsync range %d-%dkHz\n",
+			  monitor->hsync[i].lo, monitor->hsync[i].hi,
+			  mon_range->min_h, mon_range->max_h);
+		    }
+		}
+	    }
+
+	    if (monitor->nVrefresh > 0) {
+		for (i=0; i<monitor->nVrefresh; i++) {
+		    if ((1.0 - DDC_SYNC_TOLERANCE) * mon_range->min_v >
+				monitor->vrefresh[0].lo ||
+			(1.0 + DDC_SYNC_TOLERANCE) * mon_range->max_v <
+				monitor->vrefresh[0].hi) {
+			xf86DrvMsg(scrp->scrnIndex, X_WARNING,
+			  "config file vrefresh range %g-%gHz not within DDC "
+			  "vrefresh range %d-%dHz\n",
+			  monitor->vrefresh[i].lo, monitor->vrefresh[i].hi,
+			  mon_range->min_v, mon_range->max_v);
+		    }
+		}
+	    }
+        } /* if (mon_range) */
+    }
+
     /*
      * If requested by the driver, allow missing hsync and/or vrefresh ranges
      * in the monitor section.
@@ -1126,11 +1284,17 @@ xf86ValidateModes(ScrnInfoPtr scrp, DisplayModePtr availModes,
 	strategy &= ~LOOKUP_OPTIONAL_TOLERANCES;
     } else {
 	if (scrp->monitor->nHsync <= 0) {
+	    if (mon_range) {
+		scrp->monitor->hsync[0].lo = mon_range->min_h;
+		scrp->monitor->hsync[0].hi = mon_range->max_h;
+	    } else {
+		scrp->monitor->hsync[0].lo = 28;
+		scrp->monitor->hsync[0].hi = 33;
+	    }
 	    xf86DrvMsg(scrp->scrnIndex, X_WARNING,
-			"%s: Using default hsync range of 28-33kHz\n",
-			scrp->monitor->id);
-	    scrp->monitor->hsync[0].lo = 28;
-	    scrp->monitor->hsync[0].hi = 33;
+		       "%s: Using default hsync range of %.2f-%.2fkHz\n",
+		       scrp->monitor->id,
+		       scrp->monitor->hsync[0].lo, scrp->monitor->hsync[0].hi);
 	    scrp->monitor->nHsync = 1;
 	} else {
 	  for (i = 0; i < scrp->monitor->nHsync; i++)
@@ -1147,11 +1311,18 @@ xf86ValidateModes(ScrnInfoPtr scrp, DisplayModePtr availModes,
 			 scrp->monitor->hsync[i].hi);
 	}
 	if (scrp->monitor->nVrefresh <= 0) {
+	    if (mon_range) {
+		scrp->monitor->vrefresh[0].lo = mon_range->min_v;
+		scrp->monitor->vrefresh[0].hi = mon_range->max_v;
+	    } else {
+		scrp->monitor->vrefresh[0].lo = 43;
+		scrp->monitor->vrefresh[0].hi = 72;
+	    }
 	    xf86DrvMsg(scrp->scrnIndex, X_WARNING,
-			"%s: using default vrefresh range of 43-72Hz\n",
-			scrp->monitor->id);
-	    scrp->monitor->vrefresh[0].lo = 43;
-	    scrp->monitor->vrefresh[0].hi = 72;
+		       "%s: using default vrefresh range of %.2f-%.2fHz\n",
+		       scrp->monitor->id,
+		       scrp->monitor->vrefresh[0].lo,
+		       scrp->monitor->vrefresh[0].hi);
 	    scrp->monitor->nVrefresh = 1;
 	} else {
 	  for (i = 0; i < scrp->monitor->nVrefresh; i++)
@@ -1192,18 +1363,10 @@ xf86ValidateModes(ScrnInfoPtr scrp, DisplayModePtr availModes,
     else
 	BankFormat = xf86GetPixFormat(scrp, 1);	/* >not< scrp->depth! */
 
-    bitsPerPixel = scrp->fbFormat.bitsPerPixel;
-    pixmapPad = scrp->fbFormat.scanlinePad;
-    if (scrp->depth == 4)
-	pixmapPad *= 4;		/* 4 planes */
-
     if (scrp->xInc <= 0)
         scrp->xInc = 8;		/* Suitable for VGA and others */
 
 #define _VIRTUALX(x) ((((x) + scrp->xInc - 1) / scrp->xInc) * scrp->xInc)
-#define _VIDEOSIZE(w, x, y) \
-    ((((((w) * bitsPerPixel) + pixmapPad - 1) / pixmapPad) * pixmapPad * \
-     ((y) - 1)) + ((x) * bitsPerPixel))
 
     /*
      * Determine maxPitch if it wasn't given explicitly.  Note linePitches
@@ -1263,7 +1426,7 @@ xf86ValidateModes(ScrnInfoPtr scrp, DisplayModePtr availModes,
 	    return -1;
 	}
 
-	if (_VIDEOSIZE(linePitch, virtualX, virtualY) > pixelArea) {
+	if (!xf86CheckModeSize(scrp, linePitch, virtualX, virtualY)) {
 	    xf86DrvMsg(scrp->scrnIndex, X_ERROR,
 		      "Virtual size (%dx%d) (pitch %d) exceeds video memory\n",
 		      virtualX, virtualY, linePitch);
@@ -1290,6 +1453,10 @@ xf86ValidateModes(ScrnInfoPtr scrp, DisplayModePtr availModes,
 	    status = xf86InitialCheckModeForDriver(scrp, p, clockRanges,
 						   strategy, maxPitch,
 						   virtualX, virtualY);
+
+	    if (status == MODE_OK)
+		status = xf86CheckModeForMonitor(p, scrp->monitor);
+
 	    if (status == MODE_OK) {
 		new = xnfalloc(sizeof(DisplayModeRec));
 		*new = *p;
@@ -1339,8 +1506,10 @@ xf86ValidateModes(ScrnInfoPtr scrp, DisplayModePtr availModes,
     last = NULL;
     if (modeNames != NULL) {
 	for (i = 0; modeNames[i] != NULL; i++) {
+	    userModes = TRUE;
 	    new = xnfcalloc(1, sizeof(DisplayModeRec));
 	    new->prev = last;
+	    new->type = M_T_USERDEF;
 	    new->name = xnfalloc(strlen(modeNames[i]) + 1);
 	    strcpy(new->name, modeNames[i]);
 	    if (new->prev)
@@ -1349,8 +1518,12 @@ xf86ValidateModes(ScrnInfoPtr scrp, DisplayModePtr availModes,
 	    endp = &new->next;
 	}
     }
+
     /* Lookup each mode */
+    validateAllDefaultModes = TRUE;
     for (p = scrp->modes; ; p = p->next) {
+	Bool repeat;
+
 	/*
 	 * If the supplied mode names don't produce a valid mode, scan through
 	 * unconsidered modePool members until one survives validation.  This
@@ -1358,9 +1531,10 @@ xf86ValidateModes(ScrnInfoPtr scrp, DisplayModePtr availModes,
 	 */
 
 	if (p == NULL) {
-	    if (numModes > 0)
+	    if ((numModes > 0) && !validateAllDefaultModes)
 		break;
 
+	    validateAllDefaultModes = TRUE;
 	    r = NULL;
 	    modeSize = 0;
 	    for (q = scrp->modePool;  q != NULL;  q = q->next) {
@@ -1377,9 +1551,20 @@ xf86ValidateModes(ScrnInfoPtr scrp, DisplayModePtr availModes,
 
 		    if (p != NULL)
 			q->prev = p;
-		    else if (modeSize < (q->HDisplay * q->VDisplay)) {
-			r = q;
-			modeSize = q->HDisplay * q->VDisplay;
+		    else {
+			/*
+			 * A quick check to not allow default modes with
+			 * horizontal timing parameters that CRTs may have
+			 * problems with.
+			 */
+			if ((q->type & M_T_DEFAULT) &&
+			    ((double)q->HTotal / (double)q->HDisplay) < 1.15)
+			    continue;
+
+			if (modeSize < (q->HDisplay * q->VDisplay)) {
+			    r = q;
+			    modeSize = q->HDisplay * q->VDisplay;
+			}
 		    }
 		}
 	    }
@@ -1390,6 +1575,8 @@ xf86ValidateModes(ScrnInfoPtr scrp, DisplayModePtr availModes,
 	    p = xnfcalloc(1, sizeof(DisplayModeRec));
 	    p->prev = last;
 	    p->name = xnfalloc(strlen(r->name) + 1);
+	    if (!userModes)
+		p->type = M_T_USERDEF;
 	    strcpy(p->name, r->name);
 	    if (p->prev)
 		p->prev->next = p;
@@ -1397,8 +1584,27 @@ xf86ValidateModes(ScrnInfoPtr scrp, DisplayModePtr availModes,
 	    endp = &p->next;
 	}
 
+	repeat = FALSE;
     lookupNext:
+	if (repeat && ((status = p->status) != MODE_OK)) {
+		if (p->type & M_T_BUILTIN)
+		    xf86DrvMsg(scrp->scrnIndex, X_INFO,
+			       "Not using built-in mode \"%s\" (%s)\n",
+			       p->name, xf86ModeStatusToString(status));
+		else if (p->type & M_T_DEFAULT)
+		    xf86DrvMsg(scrp->scrnIndex, X_INFO,
+			       "Not using default mode \"%s\" (%s)\n", p->name,
+			       xf86ModeStatusToString(status));
+		else
+		    xf86DrvMsg(scrp->scrnIndex, X_INFO,
+			       "Not using mode \"%s\" (%s)\n", p->name,
+			       xf86ModeStatusToString(status));
+	}
+	saveType = p->type;
 	status = xf86LookupMode(scrp, p, clockRanges, strategy);
+	if (repeat && status == MODE_NOMODE) {
+	    continue;
+	}
 	if (status != MODE_OK) {
 		if (p->type & M_T_BUILTIN)
 		    xf86DrvMsg(scrp->scrnIndex, X_INFO,
@@ -1423,11 +1629,26 @@ xf86ValidateModes(ScrnInfoPtr scrp, DisplayModePtr availModes,
 		p->status = status;
 	    continue;
 	}
+	p->type |= saveType;
+	repeat = TRUE;
 
 	newLinePitch = linePitch;
 	newVirtX = virtX;
 	newVirtY = virtY;
 
+	/*
+	 * Don't let non-user defined modes increase the virtual size
+	 */
+	if (!(p->type & M_T_USERDEF)) {
+	    if (p->HDisplay > virtX) {
+		p->status = MODE_VIRTUAL_X;
+		goto lookupNext;
+	    }
+	    if (p->VDisplay > virtY) {
+		p->status = MODE_VIRTUAL_Y;
+		goto lookupNext;
+	    }
+	}
 	/*
 	 * Adjust virtual width and height if the mode is too large for the
 	 * current values and if they are not fixed.
@@ -1474,7 +1695,7 @@ xf86ValidateModes(ScrnInfoPtr scrp, DisplayModePtr availModes,
 	     * Check that the pixel area required by the new virtual height
 	     * and line pitch isn't too large.
 	     */
-	    if (_VIDEOSIZE(newLinePitch, newVirtX, newVirtY) > pixelArea) {
+	    if (!xf86CheckModeSize(scrp, newLinePitch, newVirtX, newVirtY)) {
 		p->status = MODE_MEM_VIRT;
 		goto lookupNext;
 	    }
@@ -1491,20 +1712,7 @@ xf86ValidateModes(ScrnInfoPtr scrp, DisplayModePtr availModes,
 	    p->status = (scrp->ValidMode)(scrp->scrnIndex, p, FALSE,
 					  MODECHECK_FINAL);
 
-	   if (p->status != MODE_OK) {
-	        if (p->type & M_T_BUILTIN)
-		    xf86DrvMsg(scrp->scrnIndex, X_INFO,
-			       "Not using built-in mode \"%s\" (%s)\n",
-			       p->name, xf86ModeStatusToString(p->status));
-		else if (p->type & M_T_DEFAULT)
-		    xf86DrvMsg(scrp->scrnIndex, X_INFO,
-			       "Not using default mode \"%s\" (%s)\n", p->name,
-			       xf86ModeStatusToString(p->status));
-		else
-		    xf86DrvMsg(scrp->scrnIndex, X_INFO,
-			       "Not using mode \"%s\" (%s)\n", p->name,
-			       xf86ModeStatusToString(p->status));
-
+	    if (p->status != MODE_OK) {
 	        goto lookupNext;
 	    }
 	}
@@ -1518,7 +1726,6 @@ xf86ValidateModes(ScrnInfoPtr scrp, DisplayModePtr availModes,
     }
 
 #undef _VIRTUALX
-#undef _VIDEOSIZE
 
     /* Update the ScrnInfoRec parameters */
     
@@ -1671,46 +1878,44 @@ xf86SetCrtcForModes(ScrnInfoPtr scrp, int adjustFlags)
 static void
 add(char **p, char *new)
 {
-    if (!*p) {
-        *p = xnfalloc(strlen(new) + 1);
-	strcpy(*p,new);
-    } else {
-        *p = xnfrealloc(*p,((*p)?strlen(*p):0) + strlen(new) + 2);
-	strcat(*p," ");
-	strcat(*p,new);
-    }
+    *p = xnfrealloc(*p, strlen(*p) + strlen(new) + 2);
+    strcat(*p, " ");
+    strcat(*p, new);
 }
 
 static void
 PrintModeline(int scrnIndex,DisplayModePtr mode)
 {
     char tmp[256];
-    char *flags = NULL;
+    char *flags = xnfcalloc(1, 1);
 
     if (mode->HSkew) { 
-        snprintf(tmp,256,"hskew %i",mode->HSkew); 
-	add(&flags,tmp);
+	snprintf(tmp, 256, "hskew %i", mode->HSkew); 
+	add(&flags, tmp);
     }
     if (mode->VScan) { 
-        snprintf(tmp,256,"vscan %i",mode->VScan); 
-	add(&flags,tmp);
+	snprintf(tmp, 256, "vscan %i", mode->VScan); 
+	add(&flags, tmp);
     }
-    if (mode->Flags & V_INTERLACE) add(&flags,"interlace");
-    if (mode->Flags & V_CSYNC) add(&flags,"composite");
-    if (mode->Flags & V_DBLSCAN) add(&flags,"doublescan");
-    if (mode->Flags & V_BCAST) add(&flags,"bcast");
-    if (mode->Flags & V_PHSYNC) add(&flags,"+hsync");
-    if (mode->Flags & V_NHSYNC) add(&flags,"-hsync");
-    if (mode->Flags & V_PVSYNC) add(&flags,"+vsync");
-    if (mode->Flags & V_NVSYNC) add(&flags,"-vsync");
-    if (mode->Flags & V_PCSYNC) add(&flags,"+csync");
-    if (mode->Flags & V_NCSYNC) add(&flags,"-csync");
-    xf86DrvMsgVerb(scrnIndex,X_INFO,3,
-		   "Modeline \"%s\"  %6.2f  %i %i %i %i  %i %i %i %i %s\n",
-		   mode->name,mode->Clock/1000., mode->HDisplay,
-		   mode->HSyncStart,mode->HSyncEnd,mode->HTotal,
-		   mode->VDisplay,mode->VSyncStart,mode->VSyncEnd,
-		   mode->VTotal,flags);
+    if (mode->Flags & V_INTERLACE) add(&flags, "interlace");
+    if (mode->Flags & V_CSYNC) add(&flags, "composite");
+    if (mode->Flags & V_DBLSCAN) add(&flags, "doublescan");
+    if (mode->Flags & V_BCAST) add(&flags, "bcast");
+    if (mode->Flags & V_PHSYNC) add(&flags, "+hsync");
+    if (mode->Flags & V_NHSYNC) add(&flags, "-hsync");
+    if (mode->Flags & V_PVSYNC) add(&flags, "+vsync");
+    if (mode->Flags & V_NVSYNC) add(&flags, "-vsync");
+    if (mode->Flags & V_PCSYNC) add(&flags, "+csync");
+    if (mode->Flags & V_NCSYNC) add(&flags, "-csync");
+#if 0
+    if (mode->Flags & V_CLKDIV2) add(&flags, "vclk/2");
+#endif
+    xf86DrvMsgVerb(scrnIndex, X_INFO, 3,
+		   "Modeline \"%s\"  %6.2f  %i %i %i %i  %i %i %i %i%s\n",
+		   mode->name, mode->Clock/1000., mode->HDisplay,
+		   mode->HSyncStart, mode->HSyncEnd, mode->HTotal,
+		   mode->VDisplay, mode->VSyncStart, mode->VSyncEnd,
+		   mode->VTotal, flags);
     xfree(flags);
 }
 
@@ -1719,7 +1924,7 @@ xf86PrintModes(ScrnInfoPtr scrp)
 {
     DisplayModePtr p;
     float hsync, refresh = 0;
-    char *desc, *desc2, *prefix;
+    char *desc, *desc2, *prefix, *uprefix;
 
     if (scrp == NULL)
 	return;
@@ -1762,19 +1967,28 @@ xf86PrintModes(ScrnInfoPtr scrp)
 	    prefix = "Default mode";
 	else
 	    prefix = "Mode";
+	if (p->type & M_T_USERDEF)
+	    uprefix = "*";
+	else
+	    uprefix = " ";
 	if (hsync == 0 || refresh == 0) {
-	    xf86DrvMsg(scrp->scrnIndex, X_CONFIG,
-			"%s \"%s\"\n", prefix, p->name);
+	    if (p->name)
+		xf86DrvMsg(scrp->scrnIndex, X_CONFIG,
+			   "%s%s \"%s\"\n", uprefix, prefix, p->name);
+	    else
+		xf86DrvMsg(scrp->scrnIndex, X_PROBED,
+			   "%s%s %dx%d (unnamed)\n",
+			   uprefix, prefix, p->HDisplay, p->VDisplay);
 	} else if (p->Clock == p->SynthClock) {
 	    xf86DrvMsg(scrp->scrnIndex, X_CONFIG,
-			"%s \"%s\": %.1f MHz, %.1f kHz, %.1f Hz%s%s\n",
-			prefix, p->name, p->Clock / 1000.0, hsync, refresh,
-			desc, desc2);
+			"%s%s \"%s\": %.1f MHz, %.1f kHz, %.1f Hz%s%s\n",
+			uprefix, prefix, p->name, p->Clock / 1000.0,
+			hsync, refresh, desc, desc2);
 	} else {
 	    xf86DrvMsg(scrp->scrnIndex, X_CONFIG,
-			"%s \"%s\": %.1f MHz (scaled from %.1f MHz), "
+			"%s%s \"%s\": %.1f MHz (scaled from %.1f MHz), "
 			"%.1f kHz, %.1f Hz%s%s\n",
-			prefix, p->name, p->Clock / 1000.0,
+			uprefix, prefix, p->name, p->Clock / 1000.0,
 			p->SynthClock / 1000.0, hsync, refresh, desc, desc2);
 	}
 	if (hsync != 0 && refresh != 0)
