@@ -1,7 +1,5 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/apm/apm_accel.c,v 1.16 2001/04/25 17:46:42 alanh Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/apm/apm_accel.c,v 1.21 2002/10/08 22:14:03 tsi Exp $ */
 
-
-#define COMPILER_H_EXTRAS
 #include "apm.h"
 #include "miline.h"
 
@@ -29,8 +27,6 @@ static unsigned char apmROP[] = {
 };
 
 
-static void Dump(void* start, u32 len);
-
 #include "apm_funcs.c"
 
 #define IOP_ACCESS
@@ -147,6 +143,7 @@ ApmCacheMonoStipple(ScrnInfoPtr pScrn, PixmapPtr pPix)
     return &pCache->apmStippleCache;
 }
 
+#if 0
 extern GCOps XAAPixmapOps;
 static RegionPtr (*SaveCopyAreaPixmap)(DrawablePtr, DrawablePtr, GC *, int, int, int, int, int, int);
 
@@ -203,6 +200,7 @@ ApmCopyAreaPixmap(DrawablePtr pSrcDrawable, DrawablePtr pDstDrawable, GC *pGC,
 					width, height,
 					dstx, dsty);
 }
+#endif
 
 void ApmAccelReserveSpace(ApmPtr pApm)
 {
@@ -753,20 +751,3 @@ void ApmSetupXAAInfo(ApmPtr pApm, XAAInfoRecPtr pXAAinfo)
 	}
     }
 }
-
-
-
-static void
-Dump(void* start, u32 len)
-{
-  u8* i;
-  int c = 0;
-  ErrorF("Memory Dump. Start 0x%x length %d\n", (u32)start, len);
-  for (i = (u8*)start; i < ((u8*)start+len); i++)
-  {
-    ErrorF("%02x ", *i);
-    if (c++ % 25 == 24)
-      ErrorF("\n");
-  }
-  ErrorF("\n");
-}
