@@ -1,4 +1,4 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/os-support/shared/libc_wrapper.c,v 1.74 2001/05/15 10:19:42 eich Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/os-support/shared/libc_wrapper.c,v 1.88 2003/02/22 06:00:39 dawes Exp $ */
 /*
  * Copyright 1997 by The XFree86 Project, Inc.
  *
@@ -23,6 +23,9 @@
  *
  */
 
+#if defined(linux) && !defined(__GLIBC__)
+#undef __STRICT_ANSI__
+#endif
 #include <X.h>
 #include <Xmd.h>
 #include <Xos.h>
@@ -34,18 +37,20 @@
 #endif
 #include <sys/time.h>
 #include <math.h>
+#ifdef sun
+#include <ieeefp.h>
+#endif
 #include <stdarg.h>
 #include <fcntl.h>
 #include "Xfuncproto.h"
 #include "os.h"
-#include <stdarg.h>
 #include <ctype.h>
 #include <unistd.h>
 #include <string.h>
 #include <errno.h>
 #include <stdio.h>
 #include <sys/ioctl.h>
-#ifdef __EMX__
+#ifdef __UNIXOS2__
 #define NO_MMAP
 #include <sys/param.h>
 #endif
@@ -77,11 +82,6 @@ extern int mmapFd;
 #include <stdlib.h>
 #endif
 
-#ifdef __CYGWIN__
-#define vsscanf sscanf
-#define vfscanf fscanf
-#endif
-
 #define NEED_XF86_TYPES
 #define NEED_XF86_PROTOTYPES
 #define DONT_DEFINE_WRAPPERS
@@ -117,7 +117,7 @@ void xf86WrapperInit(void);
 #endif
 typedef struct dirent DIRENTRY;
 
-#ifdef __EMX__
+#ifdef __UNIXOS2__
 #define _POSIX_SOURCE
 #endif
 #ifdef ISC202
@@ -133,7 +133,7 @@ typedef struct dirent DIRENTRY;
 #include <sys/wait.h>
 #undef _POSIX_SOURCE
 #else
-#if defined(MINIX) || defined(AMOEBA) || (defined(ISC) && defined(_POSIX_SOURCE)) || defined(Lynx) || (defined (__alpha__) && defined(linux))
+#if (defined(ISC) && defined(_POSIX_SOURCE)) || defined(Lynx) || (defined (__alpha__) && defined(linux))
 #include <sys/types.h>
 #endif
 #include <sys/wait.h>
@@ -165,6 +165,11 @@ typedef struct dirent DIRENTRY;
 #endif
 #include <setjmp.h>
 
+#if defined(setjmp) && \
+    defined(__GLIBC__) && __GLIBC__ == 2 && __GLIBC_MINOR__ < 2
+#define HAS_GLIBC_SIGSETJMP 1
+#endif
+
 #if 0
 #define SETBUF_RETURNS_INT
 #endif
@@ -399,7 +404,9 @@ xf86open(const char *path, int flags, ...)
     va_start(ap, flags);
     flags = xfToOsOpenFlags(flags);
     if (flags & O_CREAT) {
-	mode_t mode = va_arg(ap, mode_t);
+	/* can't request a mode_t directly on systems where mode_t 
+	   is an unsigned short */
+	mode_t mode = (mode_t)va_arg(ap, unsigned int);
 	fd = open(path, flags, mode);
     } else {
 	fd = open(path, flags);
@@ -473,6 +480,9 @@ xf86mmap(void *start, xf86size_t length, int prot,
     if (flags & XF86_MAP_FIXED)		f |= MAP_FIXED;
     if (flags & XF86_MAP_SHARED)	f |= MAP_SHARED;
     if (flags & XF86_MAP_PRIVATE)	f |= MAP_PRIVATE;
+#ifdef __x86_64__
+    if (flags & XF86_MAP_32BIT)	        f |= MAP_32BIT;
+#endif
     if (prot  & XF86_PROT_EXEC)		p |= PROT_EXEC;
     if (prot  & XF86_PROT_READ)		p |= PROT_READ;
     if (prot  & XF86_PROT_WRITE)	p |= PROT_WRITE;
@@ -887,7 +897,7 @@ xf86strerror(int n)
 		mapnum (ENOTDIR);
 		mapnum (EPIPE);
 		mapnum (EROFS);
-#ifndef __EMX__
+#ifndef __UNIXOS2__
 		mapnum (ETXTBSY);	/* not POSIX 1 */
 #endif
 		mapnum (ENOTTY);
@@ -1171,18 +1181,13 @@ xf86bsearch(const void *key, const void *base, xf86size_t nmemb,
 	return bsearch(key, base, (size_t)nmemb, (size_t)size, compar);
 }
 
-/*VARARGS1*/
 int
 xf86execl(const char *pathname, const char *arg, ...)
 {
-#ifndef __EMX__
+#ifndef __UNIXOS2__
     int i;
     pid_t pid;
-#ifdef MACH386
-    union wait exit_status;
-#else
     int exit_status;
-#endif
     char *arglist[5];
     va_list args;
     va_start(args, arg);
@@ -1206,7 +1211,7 @@ xf86execl(const char *pathname, const char *arg, ...)
 	xf86DisableIO();
 #endif
         setuid(getuid());
-#if !defined(SELF_CONTAINED_WRAPPER) && !defined(AMOEBA) && !defined(MINIX)
+#if !defined(SELF_CONTAINED_WRAPPER)
         /* set stdin, stdout to the consoleFD, and leave stderr alone */
         for (i = 0; i < 2; i++)
         {
@@ -1262,7 +1267,7 @@ xf86execl(const char *pathname, const char *arg, ...)
     }
 #else
     return(1);
-#endif /* __EMX__ Disable this crazy business for now */
+#endif /* __UNIXOS2__ Disable this crazy business for now */
 }
 
 void
@@ -1368,7 +1373,7 @@ xfToOsChmodMode(xf86mode_t xfmode)
 
     if (xfmode & XF86_S_ISUID) mode |= S_ISUID;
     if (xfmode & XF86_S_ISGID) mode |= S_ISGID;
-#ifndef __EMX__
+#ifndef __UNIXOS2__
     if (xfmode & XF86_S_ISVTX) mode |= S_ISVTX;
 #endif
     if (xfmode & XF86_S_IRUSR) mode |= S_IRUSR;
@@ -1397,7 +1402,7 @@ xf86chmod(const char *path, xf86mode_t xfmode)
 int
 xf86chown(const char *path, xf86uid_t owner, xf86gid_t group)
 {
-#ifndef __EMX__
+#ifndef __UNIXOS2__
     int rc = chown(path, owner, group);
 #else
     int rc = 0;
@@ -1431,7 +1436,7 @@ xfToOsMknodMode(xf86mode_t xfmode)
 
     if (xfmode & XF86_S_IFREG) mode |= S_IFREG;
     if (xfmode & XF86_S_IFCHR) mode |= S_IFCHR;
-#ifndef __EMX__
+#ifndef __UNIXOS2__
     if (xfmode & XF86_S_IFBLK) mode |= S_IFBLK;
 #endif
     if (xfmode & XF86_S_IFIFO) mode |= S_IFIFO;
@@ -1442,7 +1447,7 @@ xfToOsMknodMode(xf86mode_t xfmode)
 int xf86mknod(const char *pathname, xf86mode_t xfmode, xf86dev_t dev)
 {
     mode_t mode = xfToOsMknodMode(xfmode);
-#ifndef __EMX__
+#ifndef __UNIXOS2__
     int rc      = mknod(pathname, mode, dev);
 #else
     int rc = 0;
@@ -1540,6 +1545,17 @@ xf86fabs(double x)
         return(fabs(x));
 }
 
+int 
+xf86finite(double x)
+{
+#ifndef QNX4
+	return(finite(x));
+#else
+	/* XXX Replace this with something that really works. */
+	return 1;
+#endif
+}
+
 double
 xf86floor(double x)
 {
@@ -1825,7 +1841,7 @@ xf86GetErrno ()
 		mapnum (ENOTDIR);
 		mapnum (EPIPE);
 		mapnum (EROFS);
-#ifndef __EMX__
+#ifndef __UNIXOS2__
 		mapnum (ETXTBSY);	/* not POSIX 1 */
 #endif
 		mapnum (ENOTTY);
@@ -1868,10 +1884,16 @@ char *
 xf86shmat(int id, char *addr, int xf86shmflg)
 {
     int shmflg = 0;
-    
+
+#ifdef SHM_RDONLY
     if (xf86shmflg & XF86SHM_RDONLY) shmflg |= SHM_RDONLY;
-    if (xf86shmflg & XF86SHM_RND) shmflg    |= SHM_RND;
-    if (xf86shmflg & XF86SHM_REMAP) shmflg  |= SHM_REMAP;
+#endif
+#ifdef SHM_RND
+    if (xf86shmflg & XF86SHM_RND)    shmflg |= SHM_RND;
+#endif
+#ifdef SHM_REMAP
+    if (xf86shmflg & XF86SHM_REMAP)  shmflg |= SHM_REMAP;
+#endif
 
     return shmat(id,addr,shmflg);
 }
@@ -1929,14 +1951,39 @@ xf86shmdt(char *addr)
 #endif /* HAVE_SYSV_IPC */
 
 int
-xf86setjmp(xf86jmp_buf xf86env)
+xf86getjmptype()
 {
-    return setjmp((void *)xf86env);
+#ifdef HAS_GLIBC_SIGSETJMP
+    return 1;
+#else
+    return 0;
+#endif
 }
 
-void
-xf86longjmp(xf86jmp_buf xf86env, int val)
+#ifdef HAS_GLIBC_SIGSETJMP
+int
+xf86setjmp(xf86jmp_buf env)
 {
-    longjmp((void *)xf86env, val);
+    FatalError("setjmp: type 0 called instead of type %d\n", xf86getjmptype());
+}
+#else
+int
+xf86setjmp1(xf86jmp_buf env, int arg2)
+{
+    FatalError("setjmp: type 1 called instead of type %d\n", xf86getjmptype());
+}
+#endif
+
+int
+xf86setjmp1_arg2()
+{
+    return 0;
+}
+
+int
+xf86setjmperror(xf86jmp_buf env)
+{
+    FatalError("setjmp: don't know how to handle setjmp() type %d\n",
+	       xf86getjmptype());
 }
 
