@@ -30,13 +30,15 @@
  *		Peter Busch
  *		Harold L Hunt II
  */
-/* $XFree86: xc/programs/Xserver/hw/xwin/winpfbdd.c,v 1.4 2001/05/02 00:45:26 alanh Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xwin/winpfbdd.c,v 1.17 2002/10/17 08:18:22 alanh Exp $ */
 
 #include "win.h"
 
+
 /*
  * Create a DirectDraw primary surface 
  */
+
 Bool
 winAllocateFBPrimaryDD (ScreenPtr pScreen)
 {
@@ -47,8 +49,8 @@ winAllocateFBPrimaryDD (ScreenPtr pScreen)
   DDSURFACEDESC		*pddsdPrimary = NULL;
   DDSURFACEDESC		*pddsdOffscreen = NULL;
   RECT			rcClient;
-  
-  ErrorF ("winAllocateFBPrimaryDD ()\n");
+
+  ErrorF ("winAllocateFBPrimaryDD\n");
 
   /* Get client area location in screen coords */
   GetClientRect (pScreenPriv->hwndScreen, &rcClient);
@@ -57,41 +59,55 @@ winAllocateFBPrimaryDD (ScreenPtr pScreen)
 		   (LPPOINT)&rcClient, 2);
 
   /* Create a DirectDraw object, store the address at lpdd */
-  ddrval = DirectDrawCreate (NULL, &pScreenPriv->pdd, NULL);
+  ddrval = (*g_fpDirectDrawCreate) (NULL, &pScreenPriv->pdd, NULL);
   if (ddrval != DD_OK)
-    FatalError ("winAllocateFBPrimaryDD () - Could not start DirectDraw\n");
+    FatalError ("winAllocateFBPrimaryDD - Could not start DirectDraw\n");
+  
+  /* Get a DirectDraw2 interface pointer */
+  ddrval = IDirectDraw_QueryInterface (pScreenPriv->pdd,
+				       &IID_IDirectDraw2,
+				       (LPVOID*) &pScreenPriv->pdd2);
+  if (FAILED (ddrval))
+    {
+      ErrorF ("winAllocateFBShadowDD - Failed DD2 query: %08x\n",
+	      ddrval);
+      return FALSE;
+    }
 
-  ErrorF ("winAllocateFBPrimaryDD () - Created and initialized DD\n");
+
+  ErrorF ("winAllocateFBPrimaryDD - Created and initialized DD\n");
 
   /* Are we windowed or fullscreen? */
   if (pScreenInfo->fFullScreen)
     {
       /* Full screen mode */
-      ddrval = IDirectDraw_SetCooperativeLevel (pScreenPriv->pdd,
-						pScreenPriv->hwndScreen,
-						DDSCL_FULLSCREEN
-						| DDSCL_EXCLUSIVE);
+      ddrval = IDirectDraw2_SetCooperativeLevel (pScreenPriv->pdd2,
+						 pScreenPriv->hwndScreen,
+						 DDSCL_FULLSCREEN
+						 | DDSCL_EXCLUSIVE);
       if (FAILED (ddrval))
-	FatalError ("winAllocateFBPrimaryDD () - Could not set "\
+	FatalError ("winAllocateFBPrimaryDD - Could not set "
 		    "cooperative level\n");
 
       /* Change the video mode to the mode requested */
-      ddrval = IDirectDraw_SetDisplayMode (pScreenPriv->pdd,
-					   pScreenInfo->dwWidth,
-					   pScreenInfo->dwHeight,
-					   pScreenInfo->dwDepth);
+      ddrval = IDirectDraw2_SetDisplayMode (pScreenPriv->pdd2,
+					    pScreenInfo->dwWidth,
+					    pScreenInfo->dwHeight,
+					    pScreenInfo->dwBPP,
+					    pScreenInfo->dwRefreshRate,
+					    0);
        if (FAILED (ddrval))
-	FatalError ("winAllocateFBPrimaryDD () - Could not set "\
+	FatalError ("winAllocateFBPrimaryDD - Could not set "
 		    "full screen display mode\n");
     }
   else
     {
       /* Windowed mode */
-      ddrval = IDirectDraw_SetCooperativeLevel (pScreenPriv->pdd,
-						pScreenPriv->hwndScreen,
-						DDSCL_NORMAL);
+      ddrval = IDirectDraw2_SetCooperativeLevel (pScreenPriv->pdd2,
+						 pScreenPriv->hwndScreen,
+						 DDSCL_NORMAL);
       if (FAILED (ddrval))
-	FatalError ("winAllocateFBPrimaryDD () - Could not set "\
+	FatalError ("winAllocateFBPrimaryDD - Could not set "
 		    "cooperative level\n");
     }
 
@@ -102,21 +118,21 @@ winAllocateFBPrimaryDD (ScreenPtr pScreen)
   ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE;
   
   /* Create the primary surface */
-  ddrval = IDirectDraw_CreateSurface (pScreenPriv->pdd,
-				      &ddsd,
-				      &pScreenPriv->pddsPrimary,
-				      NULL);
+  ddrval = IDirectDraw2_CreateSurface (pScreenPriv->pdd2,
+				       &ddsd,
+				       &pScreenPriv->pddsPrimary,
+				       NULL);
   if (FAILED (ddrval))
-       FatalError ("winAllocateFBPrimaryDD () - Could not create primary "\
+       FatalError ("winAllocateFBPrimaryDD - Could not create primary "
 		  "surface %08x\n", ddrval);
 
-  ErrorF ("winAllocateFBPrimaryDD () - Created primary\n");
+  ErrorF ("winAllocateFBPrimaryDD - Created primary\n");
 
   /* Allocate a DD surface description for our screen privates */
   pddsdPrimary = pScreenPriv->pddsdPrimary
-    = xalloc (sizeof (DDSURFACEDESC));
+    = malloc (sizeof (DDSURFACEDESC));
   if (pddsdPrimary == NULL)
-    FatalError ("winAllocateFBPrimaryDD () - Could not allocate surface "\
+    FatalError ("winAllocateFBPrimaryDD - Could not allocate surface "
 		"description memory\n");
   ZeroMemory (pddsdPrimary, sizeof (*pddsdPrimary));
   pddsdPrimary->dwSize = sizeof (*pddsdPrimary);
@@ -138,65 +154,67 @@ winAllocateFBPrimaryDD (ScreenPtr pScreen)
   ddsd.dwWidth = pScreenInfo->dwWidth;
 
   /* Create the shadow surface */
-  ddrval = IDirectDraw_CreateSurface (pScreenPriv->pdd,
-				      &ddsd,
-				      &pScreenPriv->pddsOffscreen,
-				      NULL);
+  ddrval = IDirectDraw2_CreateSurface (pScreenPriv->pdd2,
+				       &ddsd,
+				       &pScreenPriv->pddsOffscreen,
+				       NULL);
   if (ddrval != DD_OK)
-    FatalError ("winAllocateFBPrimaryDD () - Could not create shadow "\
+    FatalError ("winAllocateFBPrimaryDD - Could not create shadow "
 		"surface\n");
   
-  ErrorF ("winAllocateFBPrimaryDD () - Created offscreen\n");
+  ErrorF ("winAllocateFBPrimaryDD - Created offscreen\n");
 
   /* Allocate a DD surface description for our screen privates */
   pddsdOffscreen = pScreenPriv->pddsdOffscreen
-    = xalloc (sizeof (DDSURFACEDESC));
+    = malloc (sizeof (DDSURFACEDESC));
   if (pddsdOffscreen == NULL)
-    FatalError ("winAllocateFBPrimaryDD () - Could not allocate surface "\
+    FatalError ("winAllocateFBPrimaryDD - Could not allocate surface "
 		"description memory\n");
   ZeroMemory (pddsdOffscreen, sizeof (*pddsdOffscreen));
   pddsdOffscreen->dwSize = sizeof (*pddsdOffscreen);
 
-  ErrorF ("winAllocateFBPrimaryDD () - Locking primary\n");
+  ErrorF ("winAllocateFBPrimaryDD - Locking primary\n");
 
   /* Lock the primary surface */
-  ddrval = IDirectDrawSurface_Lock (pScreenPriv->pddsPrimary,
+  ddrval = IDirectDrawSurface2_Lock (pScreenPriv->pddsPrimary,
 				    pScreenInfo->fFullScreen ? NULL:&rcClient,
 				    pddsdPrimary,
 				    DDLOCK_WAIT,
 				    NULL);
   if (ddrval != DD_OK || pddsdPrimary->lpSurface == NULL)
-    FatalError ("winAllocateFBPrimaryDD () - Could not lock "\
+    FatalError ("winAllocateFBPrimaryDD - Could not lock "
 		"primary surface\n");
 
-  ErrorF ("winAllocateFBPrimaryDD () - Locked primary\n");
+  ErrorF ("winAllocateFBPrimaryDD - Locked primary\n");
 
   /* We don't know how to deal with anything other than RGB */
   if (!(pddsdPrimary->ddpfPixelFormat.dwFlags & DDPF_RGB))
-    FatalError ("winAllocateFBPrimaryDD () - Color format other than RGB\n");
+    FatalError ("winAllocateFBPrimaryDD - Color format other than RGB\n");
 
-  /* Grab the pitch, and memory pointer from the surface desc */
-  pScreenInfo->dwStrideBytes = pddsdPrimary->u.lPitch;
-  pScreenInfo->dwStride = (pScreenInfo->dwStrideBytes * 8)
-    / pScreenInfo->dwDepth;
+  /* Grab the pitch from the surface desc */
+  pScreenInfo->dwStride = (pddsdPrimary->u1.lPitch * 8)
+    / pScreenInfo->dwBPP;
+
+  /* Save the pointer to our surface memory */
   pScreenInfo->pfb = pddsdPrimary->lpSurface;
   
   /* Grab the color depth and masks from the surface description */
-  pScreenInfo->dwDepth = pddsdPrimary->ddpfPixelFormat.u.dwRGBBitCount;
   pScreenPriv->dwRedMask = pddsdPrimary->ddpfPixelFormat.u2.dwRBitMask;
   pScreenPriv->dwGreenMask = pddsdPrimary->ddpfPixelFormat.u3.dwGBitMask;
   pScreenPriv->dwBlueMask = pddsdPrimary->ddpfPixelFormat.u4.dwBBitMask;
 
-  ErrorF ("winAllocateFBPrimaryDD () - Returning\n");
+  ErrorF ("winAllocateFBPrimaryDD - Returning\n");
 
   return TRUE;
 }
 
+
 /*
  * Call the wrapped CloseScreen function.
  * 
  * Free our resources and private structures.
  */
+
 Bool
 winCloseScreenPrimaryDD (int nIndex, ScreenPtr pScreen)
 {
@@ -204,7 +222,7 @@ winCloseScreenPrimaryDD (int nIndex, ScreenPtr pScreen)
   winScreenInfo		*pScreenInfo = pScreenPriv->pScreenInfo;
   Bool			fReturn;
   
-  ErrorF ("winCloseScreenPrimaryDD () - Freeing screen resources\n");
+  ErrorF ("winCloseScreenPrimaryDD - Freeing screen resources\n");
 
   /* Flag that the screen is closed */
   pScreenPriv->fClosed = TRUE;
@@ -220,31 +238,27 @@ winCloseScreenPrimaryDD (int nIndex, ScreenPtr pScreen)
   /* Free the offscreen surface, if there is one */
   if (pScreenPriv->pddsOffscreen)
     {
-      IDirectDrawSurface_Unlock (pScreenPriv->pddsOffscreen, NULL);
-      IDirectDrawSurface_Release (pScreenPriv->pddsOffscreen);
+      IDirectDrawSurface2_Unlock (pScreenPriv->pddsOffscreen, NULL);
+      IDirectDrawSurface2_Release (pScreenPriv->pddsOffscreen);
       pScreenPriv->pddsOffscreen = NULL;
     }
 
   /* Release the primary surface, if there is one */
   if (pScreenPriv->pddsPrimary)
     {
-      IDirectDrawSurface_Unlock (pScreenPriv->pddsPrimary, NULL);
-      IDirectDrawSurface_Release (pScreenPriv->pddsPrimary);
+      IDirectDrawSurface2_Unlock (pScreenPriv->pddsPrimary, NULL);
+      IDirectDrawSurface2_Release (pScreenPriv->pddsPrimary);
       pScreenPriv->pddsPrimary = NULL;
     }
 
   /* Free the DirectDraw object, if there is one */
   if (pScreenPriv->pdd)
     {
-      IDirectDraw_RestoreDisplayMode (pScreenPriv->pdd);
-      IDirectDraw_Release (pScreenPriv->pdd);
+      IDirectDraw2_RestoreDisplayMode (pScreenPriv->pdd);
+      IDirectDraw2_Release (pScreenPriv->pdd);
       pScreenPriv->pdd = NULL;
     }
 
-  /* Redisplay the Windows cursor */
-  if (!pScreenPriv->fCursor)
-      ShowCursor (TRUE);
-
   /* Kill our window */
   if (pScreenPriv->hwndScreen)
     {
@@ -259,11 +273,12 @@ winCloseScreenPrimaryDD (int nIndex, ScreenPtr pScreen)
   pScreenInfo->pfb = NULL;
 
   /* Free the screen privates for this screen */
-  xfree ((pointer) pScreenPriv);
+  free ((pointer) pScreenPriv);
 
   return fReturn;
 }
 
+
 /*
  * Tell mi what sort of visuals we need.
  * 
@@ -271,6 +286,7 @@ winCloseScreenPrimaryDD (int nIndex, ScreenPtr pScreen)
  * handle one format at a time, I believe.  You may want
  * to verify that last sentence.
  */
+
 Bool
 winInitVisualsPrimaryDD (ScreenPtr pScreen)
 {
@@ -291,7 +307,7 @@ winInitVisualsPrimaryDD (ScreenPtr pScreen)
   else
     pScreenPriv->dwBitsPerRGB = dwBlueBits;
   
-  ErrorF ("winInitVisualsPrimaryDD () - Masks: %08x %08x %08x bpRGB: %d\n",
+  ErrorF ("winInitVisualsPrimaryDD - Masks: %08x %08x %08x bpRGB: %d\n",
 	  pScreenPriv->dwRedMask,
 	  pScreenPriv->dwGreenMask,
 	  pScreenPriv->dwBlueMask,
@@ -300,7 +316,6 @@ winInitVisualsPrimaryDD (ScreenPtr pScreen)
   /* Create a single visual according to the Windows screen depth */
   switch (pScreenInfo->dwDepth)
     {
-    case 32:
     case 24:
     case 16:
     case 15:
@@ -312,7 +327,7 @@ winInitVisualsPrimaryDD (ScreenPtr pScreen)
 				     pScreenPriv->dwGreenMask,
 				     pScreenPriv->dwBlueMask))
 	{
-	  ErrorF ("winInitVisualsPrimaryDD () - " \
+	  ErrorF ("winInitVisualsPrimaryDD - " 
 		  "miSetVisualTypesAndMasks failed\n");
 	  return FALSE;
 	}
@@ -320,7 +335,7 @@ winInitVisualsPrimaryDD (ScreenPtr pScreen)
 
     case 8:
 #if CYGDEBUG
-      ErrorF ("winInitVisuals () - Calling miSetVisualTypesAndMasks\n");
+      ErrorF ("winInitVisuals - Calling miSetVisualTypesAndMasks\n");
 #endif /* CYGDEBUG */
       if (!miSetVisualTypesAndMasks (pScreenInfo->dwDepth,
 				     PseudoColorMask,
@@ -330,65 +345,72 @@ winInitVisualsPrimaryDD (ScreenPtr pScreen)
 				     pScreenPriv->dwGreenMask,
 				     pScreenPriv->dwBlueMask))
 	{
-	  ErrorF ("winInitVisualsPrimaryDD () - "\
+	  ErrorF ("winInitVisualsPrimaryDD - "
 		  "miSetVisualTypesAndMasks failed\n");
 	  return FALSE;
 	}
 #if CYGDEBUG
-      ErrorF ("winInitVisualsPrimaryDD () - Returned from "\
+      ErrorF ("winInitVisualsPrimaryDD - Returned from "
 	      "miSetVisualTypesAndMasks\n");
 #endif /* CYGDEBUG */
       break;
 
     default:
-      ErrorF ("winInitVisualsPrimaryDD () - Unknown screen depth\n");
+      ErrorF ("winInitVisualsPrimaryDD - Unknown screen depth\n");
       return FALSE;
     }
 
-  /* Set DPI info */
-  pScreenInfo->dwDPIx = 100;
-  pScreenInfo->dwDPIy = 100;
-
-  ErrorF ("winInitVisualsPrimaryDD () - Returning\n");
+  ErrorF ("winInitVisualsPrimaryDD - Returning\n");
 
   return TRUE;
 }
 
+
 Bool
 winAdjustVideoModePrimaryDD (ScreenPtr pScreen)
 {
   winScreenPriv(pScreen);
   winScreenInfo		*pScreenInfo = pScreenPriv->pScreenInfo;
   HDC			hdc = NULL;
-  DWORD			dwDepth;
-
-  /* Are we fullscreen? */
-  if (pScreenInfo->fFullScreen)
-    {
-      /* We don't need to adjust the video mode for fullscreen */
-      return TRUE;
-    }
+  DWORD			dwBPP;
 
   /* We're in serious trouble if we can't get a DC */
   hdc = GetDC (NULL);
   if (hdc == NULL)
     {
-      ErrorF ("winAdjustVideoModePrimaryDD () - GetDC failed\n");
+      ErrorF ("winAdjustVideoModePrimaryDD - GetDC failed\n");
       return FALSE;
     }
 
   /* Query GDI for current display depth */
-  dwDepth = GetDeviceCaps (hdc, BITSPIXEL);
+  dwBPP = GetDeviceCaps (hdc, BITSPIXEL);
 
-  /* Is GDI using a depth different than command line parameter? */
-  if (dwDepth != pScreenInfo->dwDepth)
+  /* DirectDraw can only change the depth in fullscreen mode */
+  if (pScreenInfo->dwBPP == WIN_DEFAULT_BPP)
     {
-      /* Warn user if GDI depth is different than depth specified */
-      ErrorF ("winAdjustVideoModePrimaryDD () - Command line depth: %d, "\
-	      "using depth: %d\n", pScreenInfo->dwDepth, dwDepth);
+      /* No -depth parameter passed, let the user know the depth being used */
+      ErrorF ("winAdjustVideoModePrimaryDD - Using Windows display "
+	      "depth of %d bits per pixel\n", dwBPP);
+
+      /* Use GDI's depth */
+      pScreenInfo->dwBPP = dwBPP;
+    }
+  else if (pScreenInfo->fFullScreen
+	   && pScreenInfo->dwBPP != dwBPP)
+    {
+      /* FullScreen, and GDI depth differs from -depth parameter */
+      ErrorF ("winAdjustVideoModePrimaryDD - FullScreen, using command "
+	      "line depth: %d\n", pScreenInfo->dwBPP);
+    }
+  else if (dwBPP != pScreenInfo->dwBPP)
+    {
+      /* Windowed, and GDI depth differs from -depth parameter */
+      ErrorF ("winAdjustVideoModePrimaryDD - Windowed, command line "
+	      "depth: %d, using depth: %d\n",
+	      pScreenInfo->dwBPP, dwBPP);
 
       /* We'll use GDI's depth */
-      pScreenInfo->dwDepth = dwDepth;
+      pScreenInfo->dwBPP = dwBPP;
     }
   
   /* Release our DC */
@@ -397,6 +419,13 @@ winAdjustVideoModePrimaryDD (ScreenPtr pScreen)
   return TRUE;
 }
 
+
+/*
+ * We need to blit our offscreen fb to
+ * the screen when we are activated, and we need to point
+ * the fb code back to the primary surface memory.
+ */
+
 Bool
 winActivateAppPrimaryDD (ScreenPtr pScreen)
 {
@@ -405,79 +434,180 @@ winActivateAppPrimaryDD (ScreenPtr pScreen)
   RECT			rcSrc, rcClient;
   HRESULT		ddrval = DD_OK;
 
-  /*
-   * We need to blit our offscreen fb to
-   * the screen when we are activated, and we need to point
-   * the fb code back to the primary surface memory.
-   */
-  if (pScreenPriv != NULL
-      && pScreenPriv->pddsPrimary != NULL
-      && pScreenPriv->pddsOffscreen != NULL
-      && pScreenPriv->fActive)
+  /* Check for errors */
+  if (pScreenPriv == NULL
+      || pScreenPriv->pddsPrimary == NULL
+      || pScreenPriv->pddsOffscreen == NULL)
+    return FALSE;
+
+  /* Check for do-nothing */
+  if (!pScreenPriv->fActive)
+    return TRUE;
+  
+  /* We are activating */
+  ddrval = IDirectDrawSurface2_IsLost (pScreenPriv->pddsOffscreen);
+  if (ddrval == DD_OK)
     {
-      /* We are activating */
-      ddrval = IDirectDrawSurface_IsLost (pScreenPriv->pddsOffscreen);
-      if (ddrval == DD_OK)
-	{
-	  IDirectDrawSurface_Unlock (pScreenPriv->pddsOffscreen,
-				     NULL);
-	  /*
-	   * We don't check for an error from Unlock, because it
-	   * doesn't matter if the Unlock failed.
-	   */
-	}
-	      
-      /* Restore both surfaces, just cause I like it that way */
-      IDirectDrawSurface_Restore (pScreenPriv->pddsOffscreen);
-      IDirectDrawSurface_Restore (pScreenPriv->pddsPrimary);
-			      
-      /* Get client area in screen coords */
-      GetClientRect (pScreenPriv->hwndScreen, &rcClient);
-      MapWindowPoints (pScreenPriv->hwndScreen,
-		       HWND_DESKTOP,
-		       (LPPOINT)&rcClient, 2);
-	      
-      /* Setup a source rectangle */
-      rcSrc.left = 0;
-      rcSrc.top = 0;
-      rcSrc.right = pScreenInfo->dwWidth;
-      rcSrc.bottom = pScreenInfo->dwHeight;
-
-      ddrval = IDirectDrawSurface_Blt (pScreenPriv->pddsPrimary,
-				       &rcClient,
-				       pScreenPriv->pddsOffscreen,
-				       &rcSrc,
-				       DDBLT_WAIT,
-				       NULL);
-      if (FAILED (ddrval))
-	FatalError ("winWindowProc () - Failed blitting offscreen "\
-		    "surface to primary surface %08x\n", ddrval);
-
-      /* Lock the primary surface */
-      ddrval = IDirectDrawSurface_Lock (pScreenPriv->pddsPrimary,
-					&rcClient,
-					pScreenPriv->pddsdPrimary,
-					DDLOCK_WAIT,
-					NULL);
-      if (ddrval != DD_OK
-	  || pScreenPriv->pddsdPrimary->lpSurface == NULL)
-	FatalError ("winWindowProc () - Could not lock "\
-		    "primary surface\n");
-
-      /* Notify FB of the new memory pointer */
-      winUpdateFBPointer (pScreen,
-			  pScreenPriv->pddsdPrimary->lpSurface);
-
+      IDirectDrawSurface2_Unlock (pScreenPriv->pddsOffscreen,
+				  NULL);
       /*
-       * Register the Alt-Tab combo as a hotkey so we can copy
-       * the primary framebuffer before the display mode changes
+       * We don't check for an error from Unlock, because it
+       * doesn't matter if the Unlock failed.
        */
-      RegisterHotKey (pScreenPriv->hwndScreen, 1, MOD_ALT, 9);
     }
 
+  /* Restore both surfaces, just cause I like it that way */
+  IDirectDrawSurface2_Restore (pScreenPriv->pddsOffscreen);
+  IDirectDrawSurface2_Restore (pScreenPriv->pddsPrimary);
+			      
+  /* Get client area in screen coords */
+  GetClientRect (pScreenPriv->hwndScreen, &rcClient);
+  MapWindowPoints (pScreenPriv->hwndScreen,
+		   HWND_DESKTOP,
+		   (LPPOINT)&rcClient, 2);
+
+  /* Setup a source rectangle */
+  rcSrc.left = 0;
+  rcSrc.top = 0;
+  rcSrc.right = pScreenInfo->dwWidth;
+  rcSrc.bottom = pScreenInfo->dwHeight;
+
+  ddrval = IDirectDrawSurface2_Blt (pScreenPriv->pddsPrimary,
+				    &rcClient,
+				    pScreenPriv->pddsOffscreen,
+				    &rcSrc,
+				    DDBLT_WAIT,
+				    NULL);
+  if (ddrval != DD_OK)
+    FatalError ("winActivateAppPrimaryDD () - Failed blitting offscreen "
+		"surface to primary surface %08x\n", ddrval);
+  
+  /* Lock the primary surface */
+  ddrval = IDirectDrawSurface2_Lock (pScreenPriv->pddsPrimary,
+				     &rcClient,
+				     pScreenPriv->pddsdPrimary,
+				     DDLOCK_WAIT,
+				     NULL);
+  if (ddrval != DD_OK
+      || pScreenPriv->pddsdPrimary->lpSurface == NULL)
+    FatalError ("winActivateAppPrimaryDD () - Could not lock "
+		"primary surface\n");
+
+  /* Notify FB of the new memory pointer */
+  winUpdateFBPointer (pScreen,
+		      pScreenPriv->pddsdPrimary->lpSurface);
+
+  /*
+   * Register the Alt-Tab combo as a hotkey so we can copy
+   * the primary framebuffer before the display mode changes
+   */
+  RegisterHotKey (pScreenPriv->hwndScreen, 1, MOD_ALT, 9);
+
   return TRUE;
 }
 
+
+/*
+ * Handle the Alt+Tab hotkey.
+ *
+ * We need to save the primary fb to an offscreen fb when
+ * we get deactivated, and point the fb code at the offscreen
+ * fb for the duration of the deactivation.
+ */
+
+Bool
+winHotKeyAltTabPrimaryDD (ScreenPtr pScreen)
+{
+  winScreenPriv(pScreen);
+  winScreenInfo		*pScreenInfo = pScreenPriv->pScreenInfo;
+  RECT			rcClient, rcSrc;
+  HRESULT		ddrval = DD_OK;
+
+  ErrorF ("\nwinHotKeyAltTabPrimaryDD\n\n");
+
+  /* Alt+Tab was pressed, we will lose focus very soon */
+  pScreenPriv->fActive = FALSE;
+  
+  /* Check for error conditions */
+  if (pScreenPriv->pddsPrimary == NULL
+      || pScreenPriv->pddsOffscreen == NULL)
+    return FALSE;
+
+  /* Get client area in screen coords */
+  GetClientRect (pScreenPriv->hwndScreen, &rcClient);
+  MapWindowPoints (pScreenPriv->hwndScreen,
+		   HWND_DESKTOP,
+		   (LPPOINT)&rcClient, 2);
+
+  /* Did we loose the primary surface? */
+  ddrval = IDirectDrawSurface2_IsLost (pScreenPriv->pddsPrimary);
+  if (ddrval == DD_OK)
+    {
+      ddrval = IDirectDrawSurface2_Unlock (pScreenPriv->pddsPrimary,
+					   NULL);
+      if (FAILED (ddrval))
+	FatalError ("winHotKeyAltTabPrimaryDD - Failed unlocking primary "
+		    "surface\n");
+    }
+
+  /* Setup a source rectangle */
+  rcSrc.left = 0;
+  rcSrc.top = 0;
+  rcSrc.right = pScreenInfo->dwWidth;
+  rcSrc.bottom = pScreenInfo->dwHeight;
+
+      /* Blit the primary surface to the offscreen surface */
+  ddrval = IDirectDrawSurface2_Blt (pScreenPriv->pddsOffscreen,
+				    NULL, /* should be rcDest */
+				    pScreenPriv->pddsPrimary,
+				    NULL,
+				    DDBLT_WAIT,
+				    NULL);
+  if (ddrval == DDERR_SURFACELOST)
+    {
+      IDirectDrawSurface2_Restore (pScreenPriv->pddsOffscreen);  
+      IDirectDrawSurface2_Restore (pScreenPriv->pddsPrimary);
+		  		  
+      /* Blit the primary surface to the offscreen surface */
+      ddrval = IDirectDrawSurface2_Blt (pScreenPriv->pddsOffscreen,
+					NULL,
+					pScreenPriv->pddsPrimary,
+					NULL,
+					DDBLT_WAIT,
+					NULL);
+      if (FAILED (ddrval))
+	FatalError ("winHotKeyAltTabPrimaryDD - Failed blitting primary "
+		    "surface to offscreen surface: %08x\n",
+		    ddrval);
+    }
+  else
+    {
+      FatalError ("winHotKeyAltTabPrimaryDD - Unknown error from "
+		  "Blt: %08dx\n", ddrval);
+    }
+
+  /* Lock the offscreen surface */
+  ddrval = IDirectDrawSurface2_Lock (pScreenPriv->pddsOffscreen,
+				     NULL,
+				     pScreenPriv->pddsdOffscreen,
+				     DDLOCK_WAIT,
+				     NULL);
+  if (ddrval != DD_OK
+      || pScreenPriv->pddsdPrimary->lpSurface == NULL)
+    FatalError ("winHotKeyAltTabPrimaryDD - Could not lock "
+		"offscreen surface\n");
+
+  /* Notify FB of the new memory pointer */
+  winUpdateFBPointer (pScreen,
+		      pScreenPriv->pddsdOffscreen->lpSurface);
+
+  /* Unregister our hotkey */
+  UnregisterHotKey (pScreenPriv->hwndScreen, 1);
+
+  return TRUE;
+}
+
+
 /* Set engine specific functions */
 Bool
 winSetEngineFunctionsPrimaryDD (ScreenPtr pScreen)
@@ -489,8 +619,6 @@ winSetEngineFunctionsPrimaryDD (ScreenPtr pScreen)
   pScreenPriv->pwinAllocateFB = winAllocateFBPrimaryDD;
   pScreenPriv->pwinShadowUpdate
     = (winShadowUpdateProcPtr) (void (*)())NoopDDA;
-  pScreenPriv->pwinShadowWindow
-    = (winShadowWindowProcPtr) (void (*)())NoopDDA;
   pScreenPriv->pwinCloseScreen = winCloseScreenPrimaryDD;
   pScreenPriv->pwinInitVisuals = winInitVisualsPrimaryDD;
   pScreenPriv->pwinAdjustVideoMode = winAdjustVideoModePrimaryDD;
@@ -502,6 +630,7 @@ winSetEngineFunctionsPrimaryDD (ScreenPtr pScreen)
   pScreenPriv->pwinBltExposedRegions
     = (winBltExposedRegionsProcPtr) (void (*)())NoopDDA;
   pScreenPriv->pwinActivateApp = winActivateAppPrimaryDD;
+  pScreenPriv->pwinHotKeyAltTab = winHotKeyAltTabPrimaryDD;
 
   return TRUE;
 }
