@@ -1,9 +1,13 @@
-/* $Xorg: Login.c,v 1.3 2000/08/17 19:54:17 cpqbld Exp $ */
+/* $Xorg: Login.c,v 1.4 2001/02/09 02:05:41 xorgcvs Exp $ */
 /*
 
 Copyright 1988, 1998  The Open Group
 
-All Rights Reserved.
+Permission to use, copy, modify, distribute, and sell this software and its
+documentation for any purpose is hereby granted without fee, provided that
+the above copyright notice appear in all copies and that both that
+copyright notice and this permission notice appear in supporting
+documentation.
 
 The above copyright notice and this permission notice shall be included
 in all copies or substantial portions of the Software.
@@ -22,7 +26,7 @@ other dealings in this Software without prior written authorization
 from The Open Group.
 
 */
-/* $XFree86: xc/programs/xdm/greeter/Login.c,v 3.12 2001/01/17 23:45:25 dawes Exp $ */
+/* $XFree86: xc/programs/xdm/greeter/Login.c,v 3.16 2002/10/06 20:42:16 herrb Exp $ */
 
 /*
  * xdm - display manager daemon
@@ -945,13 +949,10 @@ InsertChar (
 	MoveToBegining(ctxw, event, params, num_params);
 	return;
     default:
-	if (IsFunctionKey(keysym) || IsMiscFunctionKey(keysym) ||
-	    IsPrivateKeypadKey(keysym) || IsCursorKey(keysym) ||
-	    IsKeypadKey(keysym) || IsPFKey(keysym)) {
-	    XBell(XtDisplay(ctxw), 60);
+	if (len == 0) {
+	    if (!IsModifierKey(keysym))  /* it's not a modifier */
+		XBell(XtDisplay(ctxw), 60);
 	    return;
-	} else if (len == 0 && IsModifierKey(keysym)) {
-	    return; /* it's a modifier */
 	} else
 	    break;
     }
@@ -968,8 +969,8 @@ InsertChar (
 #endif /* XPM */
 	    len = NAME_LEN - strlen(ctx->login.data.name) - 2;
     case GET_PASSWD:
-	if (len + (int)strlen(ctx->login.data.passwd) >= NAME_LEN - 1)
-	    len = NAME_LEN - strlen(ctx->login.data.passwd) - 2;
+	if (len + (int)strlen(ctx->login.data.passwd) >= PASSWORD_LEN - 1)
+	    len = PASSWORD_LEN - strlen(ctx->login.data.passwd) - 2;
     }
 #ifndef XPM
     if (len == 0)
@@ -1254,7 +1255,7 @@ static void Destroy (Widget gw)
 {
     LoginWidget w = (LoginWidget)gw;
     bzero (w->login.data.name, NAME_LEN);
-    bzero (w->login.data.passwd, NAME_LEN);
+    bzero (w->login.data.passwd, PASSWORD_LEN);
     XtReleaseGC(gw, w->login.textGC);
     XtReleaseGC(gw, w->login.bgGC);
     XtReleaseGC(gw, w->login.xorGC);
