@@ -1,9 +1,13 @@
-/* $Xorg: PProcess.c,v 1.5 2000/08/17 19:54:59 cpqbld Exp $ */
+/* $Xorg: PProcess.c,v 1.6 2001/02/09 02:05:57 xorgcvs Exp $ */
 /*
 
 Copyright 1996, 1998  The Open Group
 
-All Rights Reserved.
+Permission to use, copy, modify, distribute, and sell this software and its
+documentation for any purpose is hereby granted without fee, provided that
+the above copyright notice appear in all copies and that both that
+copyright notice and this permission notice appear in supporting
+documentation.
 
 The above copyright notice and this permission notice shall be included
 in all copies or substantial portions of the Software.
@@ -22,6 +26,7 @@ other dealings in this Software without prior written authorization from
 The Open Group.
 
 */
+/* $XFree86: xc/programs/xrx/plugin/PProcess.c,v 1.6 2001/12/14 20:02:17 dawes Exp $ */
 
 #include "RxPlugin.h"
 #include "XUrls.h"
@@ -80,7 +85,7 @@ GetWindowGeometry(
     if (sizehints) {
 	XGetWMNormalHints (dpy, win, sizehints, &mask);
 
-	if (mask & USPosition|PPosition) {
+	if (mask & (USPosition|PPosition)) {
 	    *x = sizehints->x;
 	    *y = sizehints->y;
 	    *width = sizehints->width;
