@@ -1,9 +1,13 @@
-/* $Xorg: log.h,v 1.3 2000/08/17 19:55:05 cpqbld Exp $ */
+/* $Xorg: log.h,v 1.4 2001/02/09 02:06:01 xorgcvs Exp $ */
 /******************************************************************************
 
 Copyright 1994, 1998  The Open Group
 
-All Rights Reserved.
+Permission to use, copy, modify, distribute, and sell this software and its
+documentation for any purpose is hereby granted without fee, provided that
+the above copyright notice appear in all copies and that both that
+copyright notice and this permission notice appear in supporting
+documentation.
 
 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.
@@ -19,9 +23,16 @@ Except as contained in this notice, the name of The Open Group shall not be
 used in advertising or otherwise to promote the sale, use or other dealings
 in this Software without prior written authorization from The Open Group.
 ******************************************************************************/
-/* $XFree86: xc/programs/xsm/log.h,v 1.3 1999/03/07 14:23:40 dawes Exp $ */
+/* $XFree86: xc/programs/xsm/log.h,v 1.6 2001/12/14 20:02:26 dawes Exp $ */
+
+#ifndef _LOG_H_
+#define _LOG_H_
+
+#include <X11/Intrinsic.h>
 
 extern void DisplayLogXtProc(Widget w, XtPointer client_data, 
 			     XtPointer callData);
 extern void add_log_text(char *str);
 extern void create_log_popup(void);
+
+#endif
