@@ -1,4 +1,4 @@
-/* $XFree86: xc/programs/xvidtune/xvidtune.c,v 3.26 2000/09/26 15:57:27 tsi Exp $ */
+/* $XFree86: xc/programs/xvidtune/xvidtune.c,v 3.33 2003/02/22 04:08:44 tsi Exp $ */
 
 /*
 
@@ -53,6 +53,7 @@ int MajorVersion, MinorVersion;
 int EventBase, ErrorBase;
 int dot_clock, mode_flags;
 unsigned long    TestTimeout=5000;  /* Default test timeout */
+XtSignalId sigId;
 
 /* Minimum extension version required */
 #define MINMAJOR 0
@@ -158,6 +159,12 @@ CleanUp(Display *dpy)
 
 static void
 CatchSig(int signal)
+{
+    XtNoticeSignal(sigId);
+}
+
+static void
+CatchXtSig(XtPointer closure, XtSignalId *id)
 {
     CleanUp(XtDisplay(Top));
     exit(3);
@@ -219,6 +226,14 @@ GetMonitor (Display* dpy, int scrn)
     return TRUE;
 }
 
+static Bool
+ModeSettable(void)
+{
+    if (AppRes.field[HTotal].val == 0 || AppRes.field[VTotal].val == 0)
+	return FALSE;
+    return TRUE;
+}
+
 static int hitError = 0;
 static int (*xtErrorfunc)(Display *, XErrorEvent *);
 
@@ -393,6 +408,9 @@ SetLabel(fields i)
 static void
 UpdateSyncRates(Bool dolabels)
 {
+    if (!ModeSettable())
+	return;
+    
     AppRes.field[HSyncRate].val = AppRes.field[PixelClock].val * 1000 /
 				  AppRes.field[HTotal].val;
     AppRes.field[VSyncRate].val = AppRes.field[HSyncRate].val * 1000 /
@@ -754,7 +772,7 @@ FlagsEditCB (Widget w, XtPointer client, XtPointer call)
 static void
 BlankEditCB (Widget w, XtPointer client, XtPointer call)
 {
-    int i, len;
+    int len;
     char* string;
     fields findex = (fields) client;
     ScrollData* sdp = &AppRes.field[findex];
@@ -972,6 +990,17 @@ AckWarn (Widget w, XtPointer client, XtPointer call)
     XtDestroyWidget((Widget) client);
 }
 
+static void
+AckNoTune (Widget w, XtPointer client, XtPointer call)
+{
+    CleanUp(XtDisplay(w));
+#if XtSpecificationRelease < 6
+    exit (0);
+#else
+    XtAppSetExitFlag (XtWidgetToApplicationContext (w));
+#endif
+}
+
 static void
 displayWarning(Widget top)
 {
@@ -1016,7 +1045,32 @@ displayWarning(Widget top)
     
 }
 
+static void
+displayNoTune(Widget top)
+{
+    Widget w, popup, popupBox;
 
+    popup = XtCreateWidget ("Notice", formWidgetClass, top, NULL, 0);
+    popupBox = XtVaCreateManagedWidget(
+               "WarningBox",
+               boxWidgetClass,
+               popup,
+               NULL);
+
+    w = XtVaCreateManagedWidget( "NoTuneLabel",
+                                     labelWidgetClass,
+				     popupBox,
+                                     NULL);
+
+    w = XtVaCreateManagedWidget( "NoTuneOK",
+                                     commandWidgetClass,
+				     popupBox,
+                                     NULL);
+
+    XtAddCallback (w, XtNcallback, AckNoTune, (XtPointer)popup);
+
+    XtManageChild (popup);
+}
 
 #if 0
 static void
@@ -1094,7 +1148,6 @@ CreateHierarchy(Widget top)
 	};
 
     form = XtCreateWidget ("form", formWidgetClass, top, NULL, 0);
-
     for (i = 0; i < 14; i++)
 	forms[i] = XtCreateWidget (form_names[i], formWidgetClass, 
 		form, NULL, 0);
@@ -1162,7 +1215,7 @@ CreateHierarchy(Widget top)
                                      forms[7],
                                      NULL);
     XtAddCallback (w, XtNcallback, AdjustCB, (XtPointer)-VTotal);
-   
+
     (void) sprintf (buf, "%04x", AppRes.field[Flags].val);
     wids[0] = XtCreateWidget ("Flags-label", labelWidgetClass,
 		forms[8], NULL, 0);
@@ -1425,6 +1478,7 @@ usage(void)
 {
     fprintf(stderr, "Usage: xvidtune [option]\n");
     fprintf(stderr, "    where option is one of:\n");
+    fprintf(stderr, "        -show                             Print current modeline to stdout\n");
     fprintf(stderr, "        -next                             Switch to next video mode\n");
     fprintf(stderr, "        -prev                             Switch to previous video mode\n");
     fprintf(stderr, "        -unlock                           Enable mode switch hot-keys\n");
@@ -1439,8 +1493,8 @@ main (int argc, char** argv)
     Widget top;
     XtAppContext app;
     Display* dpy;
-    int suspendTime, offTime;
-
+    Bool modeSettable = TRUE;
+    
     static XtActionsRec actions[] = { { "xvidtune-quit", QuitAction },
 				      { "xvidtune-restore", RestoreAction },
 				      { "xvidtune-show", ShowAction },
@@ -1504,7 +1558,15 @@ main (int argc, char** argv)
         
 	if (argc != 2)
 		usage();
-	if (!strcmp(argv[1], "-next"))
+	if (!strcmp(argv[1], "-show")) {
+	  if (!GetModeLine(XtDisplay (top), DefaultScreen (XtDisplay (top)))) {
+	    fprintf(stderr, "Unable to get mode info\n");
+	    CleanUp(XtDisplay (top));
+	    return 2;
+	  }
+	  ShowCB(top, NULL, NULL);
+	  return 0;
+	} else if (!strcmp(argv[1], "-next"))
 	    i = 1;
 	else if (!strcmp(argv[1], "-prev"))
 	    i = -1;
@@ -1536,6 +1598,7 @@ main (int argc, char** argv)
     signal(SIGQUIT, CatchSig);
     signal(SIGTERM, CatchSig);
     signal(SIGHUP, CatchSig);
+    sigId = XtAppAddSignal(app, CatchXtSig, NULL);
 
     if (!GetModeLine(XtDisplay (top), DefaultScreen (XtDisplay (top)))) {
 	fprintf(stderr, "Unable to get mode info\n");
@@ -1565,7 +1628,13 @@ main (int argc, char** argv)
 	<FocusIn>: focus-in()\n<FocusOut>: focus-out()\n\
 	<Btn1Down>: select-start()\n");
 
-    CreateHierarchy (top);
+    if (!ModeSettable()) {
+	printf("Video are not settable on this chip\n");
+	displayNoTune(top);
+	modeSettable = FALSE;
+    } else
+	CreateHierarchy (top);
+    
 
     XtAppAddActions (app, actions, XtNumber(actions));
 
@@ -1581,7 +1650,10 @@ main (int argc, char** argv)
     (void) XSetWMProtocols (dpy, XtWindow (top), &wm_delete_window, 1);
 
     XtMapWidget (top);
-    displayWarning(top);
+
+    if (modeSettable)
+	displayWarning(top);
+
     /* really we should run our own event dispatching here until the
      * warning has been read...
      */
