@@ -1,9 +1,13 @@
-/* $Xorg: xwud.c,v 1.4 2000/08/17 19:55:17 cpqbld Exp $ */
+/* $Xorg: xwud.c,v 1.5 2001/02/09 02:06:04 xorgcvs Exp $ */
 /*
 
 Copyright 1985, 1986, 1988, 1998  The Open Group
 
-All Rights Reserved.
+Permission to use, copy, modify, distribute, and sell this software and its
+documentation for any purpose is hereby granted without fee, provided that
+the above copyright notice appear in all copies and that both that
+copyright notice and this permission notice appear in supporting
+documentation.
 
 The above copyright notice and this permission notice shall be included
 in all copies or substantial portions of the Software.
@@ -22,7 +26,7 @@ other dealings in this Software without prior written authorization
 from The Open Group.
 
 */
-/* $XFree86: xc/programs/xwud/xwud.c,v 3.4 1999/03/07 11:41:38 dawes Exp $ */
+/* $XFree86: xc/programs/xwud/xwud.c,v 3.7 2001/12/14 20:02:35 dawes Exp $ */
 
 /* xwud - marginally useful raster image undumper */
 
@@ -36,13 +40,7 @@ from The Open Group.
 #define  XK_LATIN1
 #include <X11/keysymdef.h>
 #include <errno.h>
-
-#ifdef X_NOT_STDC_ENV
-extern int errno;
-extern char *malloc();
-#else
 #include <stdlib.h>
-#endif
 
 Atom wm_protocols;
 Atom wm_delete_window;
